@@ -1,6 +1,7 @@
+
 /* pngtest.c - a simple test program to test libpng
  *
- * Last changed in libpng 1.5.22 [March 26, 2015]
+ * Last changed in libpng 1.6.19 [November 12, 2015]
  * Copyright (c) 1998-2015 Glenn Randers-Pehrson
  * (Version 0.96 Copyright (c) 1996, 1997 Andreas Dilger)
  * (Version 0.88 Copyright (c) 1995, 1996 Guy Eric Schalnat, Group 42, Inc.)
@@ -792,7 +793,7 @@ write_chunks(png_structp write_ptr, int location)
  */
 #ifdef PNG_TEXT_SUPPORTED
 static void
-pngtest_check_text_support(png_structp png_ptr, png_textp text_ptr,
+pngtest_check_text_support(png_const_structp png_ptr, png_textp text_ptr,
    int num_text)
 {
    while (num_text > 0)
@@ -1100,7 +1101,6 @@ test_one_file(PNG_CONST char *inname, PNG_CONST char *outname)
          png_set_gAMA_fixed(write_ptr, write_info_ptr, gamma);
    }
 #endif
-
 #else /* Use floating point versions */
 #ifdef PNG_FLOATING_POINT_SUPPORTED
 #ifdef PNG_cHRM_SUPPORTED
@@ -1116,7 +1116,6 @@ test_one_file(PNG_CONST char *inname, PNG_CONST char *outname)
       }
    }
 #endif
-
 #ifdef PNG_gAMA_SUPPORTED
    {
       double gamma;
@@ -1256,8 +1255,8 @@ test_one_file(PNG_CONST char *inname, PNG_CONST char *outname)
              scal_height);
       }
    }
-#endif
-#endif
+#endif /* FIXED_POINT */
+#endif /* FLOATING_POINT */
 #endif /* sCAL */
 
 #ifdef PNG_sPLT_SUPPORTED
@@ -1298,7 +1297,7 @@ test_one_file(PNG_CONST char *inname, PNG_CONST char *outname)
          png_set_text(write_ptr, write_info_ptr, text_ptr, num_text);
       }
    }
-#endif
+#endif /* TEXT */
 
 #ifdef PNG_tIME_SUPPORTED
    {
@@ -1308,20 +1307,20 @@ test_one_file(PNG_CONST char *inname, PNG_CONST char *outname)
       {
          png_set_tIME(write_ptr, write_info_ptr, mod_time);
 #ifdef PNG_TIME_RFC1123_SUPPORTED
-         /* We have to use memcpy instead of "=" because the string
-          * pointed to by png_convert_to_rfc1123() gets free'ed before
-          * we use it.
-          */
-         memcpy(tIME_string,
-                png_convert_to_rfc1123(read_ptr, mod_time),
-                png_sizeof(tIME_string));
+         if (png_convert_to_rfc1123_buffer(tIME_string, mod_time) != 0)
+            tIME_string[(sizeof tIME_string) - 1] = '\0';
+
+         else
+         {
+            strncpy(tIME_string, "*** invalid time ***", (sizeof tIME_string));
+            tIME_string[(sizeof tIME_string) - 1] = '\0';
+         }
 
-         tIME_string[png_sizeof(tIME_string) - 1] = '\0';
          tIME_chunk_present++;
 #endif /* TIME_RFC1123 */
       }
    }
-#endif
+#endif /* tIME */
 
 #ifdef PNG_tRNS_SUPPORTED
    {
@@ -1344,7 +1343,7 @@ test_one_file(PNG_CONST char *inname, PNG_CONST char *outname)
                trans_color);
       }
    }
-#endif
+#endif /* tRNS */
 
 #ifdef PNG_WRITE_UNKNOWN_CHUNKS_SUPPORTED
    {
@@ -1493,15 +1492,15 @@ test_one_file(PNG_CONST char *inname, PNG_CONST char *outname)
       {
          png_set_tIME(write_ptr, write_end_info_ptr, mod_time);
 #ifdef PNG_TIME_RFC1123_SUPPORTED
-         /* We have to use memcpy instead of "=" because the string
-          * pointed to by png_convert_to_rfc1123() gets free'ed before
-          * we use it.
-          */
-         memcpy(tIME_string,
-                png_convert_to_rfc1123(read_ptr, mod_time),
-                png_sizeof(tIME_string));
+         if (png_convert_to_rfc1123_buffer(tIME_string, mod_time) != 0)
+            tIME_string[(sizeof tIME_string) - 1] = '\0';
+
+         else
+         {
+            strncpy(tIME_string, "*** invalid time ***", sizeof tIME_string);
+            tIME_string[(sizeof tIME_string)-1] = '\0';
+         }
 
-         tIME_string[png_sizeof(tIME_string) - 1] = '\0';
          tIME_chunk_present++;
 #endif /* TIME_RFC1123 */
       }
@@ -1863,8 +1862,10 @@ main(int argc, char *argv[])
                      k, (unsigned long)filters_used[k]);
 #endif
 #ifdef PNG_TIME_RFC1123_SUPPORTED
-             if (tIME_chunk_present != 0)
-                fprintf(STDERR, " tIME = %s\n", tIME_string);
+            if (tIME_chunk_present != 0)
+               fprintf(STDERR, " tIME = %s\n", tIME_string);
+
+            tIME_chunk_present = 0;
 #endif /* TIME_RFC1123 */
          }
 
@@ -2056,4 +2057,4 @@ main(void)
 #endif
 
 /* Generate a compiler error if there is an old png.h in the search path. */
-typedef png_libpng_version_1_5_24 Your_png_h_is_not_version_1_5_24;
+typedef png_libpng_version_1_6_19 Your_png_h_is_not_version_1_6_19;
