@@ -127,11 +127,18 @@ NBackupAllocLock::NBackupAllocLock(thread_db* tdbb, MemoryPool& p, BackupManager
 
 bool NBackupAllocLock::fetch(thread_db* tdbb)
 {
-	if (!backup_manager->actualizeAlloc(tdbb))
+	if (!backup_manager->actualizeAlloc(tdbb, true))
 		ERR_bugcheck_msg("Can't actualize alloc table");
 	return true;
 }
 
+void NBackupAllocLock::invalidate(thread_db* tdbb)
+{
+	backup_manager->invalidateAlloc(tdbb);
+	GlobalRWLock::invalidate(tdbb);
+	NBAK_TRACE( ("invalidate alloc table allocLock(%p)", this) );
+}
+
 /******************************** BackupManager::StateWriteGuard ******************************/
 
 BackupManager::StateWriteGuard::StateWriteGuard(thread_db* _tdbb, Jrd::WIN* wnd)
@@ -192,15 +199,16 @@ void BackupManager::openDelta()
 
 	if (database->dbb_flags & (DBB_force_write | DBB_no_fs_cache))
 	{
-		PIO_force_write(diff_file,
-			database->dbb_flags & DBB_force_write,
-			database->dbb_flags & DBB_no_fs_cache);
+		setForcedWrites(database->dbb_flags & DBB_force_write,
+						database->dbb_flags & DBB_no_fs_cache);
 	}
 }
 
 void BackupManager::closeDelta()
 {
-	if (diff_file) {
+	if (diff_file) 
+	{
+		PIO_flush(database, diff_file);
 		PIO_close(diff_file);
 		diff_file = NULL;
 	}
@@ -247,9 +255,8 @@ void BackupManager::beginBackup(thread_db* tdbb)
 	{ // logical scope
 		if (database->dbb_flags & (DBB_force_write | DBB_no_fs_cache))
 		{
-			PIO_force_write(diff_file,
-				database->dbb_flags & DBB_force_write,
-				database->dbb_flags & DBB_no_fs_cache);
+			setForcedWrites(database->dbb_flags & DBB_force_write,
+							database->dbb_flags & DBB_no_fs_cache);
 		}
 
 #ifdef UNIX
@@ -258,23 +265,23 @@ void BackupManager::beginBackup(thread_db* tdbb)
 		{
 			struct stat st;
 			PageSpace* pageSpace = database->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
-			char* func = NULL;
+			const char* func = NULL;
 			while (!func && fstat(pageSpace->file->fil_desc, &st) != 0)
 			{
-				if (errno != EINTR) 
+				if (errno != EINTR)
 					func = "fstat";
 			}
 			while (!func && fchown(diff_file->fil_desc, st.st_uid, st.st_gid) != 0)
 			{
-				if (errno != EINTR) 
+				if (errno != EINTR)
 					func = "fchown";
 			}
 			while (!func && fchmod(diff_file->fil_desc, st.st_mode) != 0)
 			{
-				if (errno != EINTR) 
+				if (errno != EINTR)
 					func = "fchmod";
 			}
-			if (func) 
+			if (func)
 			{
 				stateGuard.setSuccess();
 				Firebird::system_call_failed::raise(func);
@@ -353,6 +360,46 @@ ULONG BackupManager::getPageCount()
 }
 
 
+bool BackupManager::extendDatabase(thread_db* tdbb)
+{
+	ULONG maxPage = 0;
+	{
+		LocalAllocReadGuard localAllocGuard(this);
+		AllocItemTree::Accessor all(alloc_table);
+
+		if (all.getFirst()) {
+			do
+			{
+				const ULONG pg = all.current().db_page;
+				if (maxPage < pg)
+					maxPage = pg;
+			} while (all.getNext());
+		}
+	}
+
+	PageSpace *pgSpace = database->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
+	ULONG maxAllocPage = pgSpace->maxAlloc(database->dbb_page_size);
+	if (maxAllocPage >= maxPage)
+		return true;
+
+	if (!pgSpace->extend(tdbb, maxPage, true))
+		return false;
+
+	maxAllocPage = pgSpace->maxAlloc(database->dbb_page_size);
+	while (maxAllocPage < maxPage)
+	{
+		const USHORT ret = PIO_init_data(database, pgSpace->file, tdbb->tdbb_status_vector, 
+										 maxAllocPage, 256);
+
+		if (ret != 256)
+			return false;
+
+		maxAllocPage += ret;
+	}
+	return true;
+}
+
+
 // Merge difference file to main files (if needed) and unlink() difference
 // file then. If merge is already in progress method silently returns and
 // does nothing (so it can be used for recovery on database startup).
@@ -391,6 +438,9 @@ void BackupManager::endBackup(thread_db* tdbb, bool recover)
 				endLock.unlockWrite(tdbb);
 				return;
 			}
+
+			if (backup_state == nbak_state_stalled && !extendDatabase(tdbb))
+				status_exception::raise(tdbb->tdbb_status_vector);
 		}
 
 		// Here backup state can be changed. Need to check it again after lock
@@ -403,6 +453,13 @@ void BackupManager::endBackup(thread_db* tdbb, bool recover)
 			endLock.unlockWrite(tdbb);
 			return;
 		}
+
+		if (!extendDatabase(tdbb))
+		{
+			stateGuard.setSuccess();
+			status_exception::raise(tdbb->tdbb_status_vector);
+		}
+
 		header = (Ods::header_page*) window.win_buffer;
 
 		NBAK_TRACE(("difference file %s, current backup state is %d", diff_name.c_str(), backup_state));
@@ -439,12 +496,29 @@ void BackupManager::endBackup(thread_db* tdbb, bool recover)
 		NBAK_TRACE(("Merge. State=%d, current_scn=%d, adjusted_scn=%d",
 			backup_state, current_scn, adjusted_scn));
 
-		actualizeAlloc(tdbb);
+		{ // scope
+			LocalAllocWriteGuard localAllocGuard(this);
+
+			// In merge mode allocation table can not be changed, so we don't
+			// need to acquire global allocLock.
+			actualizeAlloc(tdbb, true);
+		}
+
+		LocalAllocReadGuard localAllocGuard(this);
+
 		NBAK_TRACE(("Merge. Alloc table is actualized."));
 		AllocItemTree::Accessor all(alloc_table);
 
-		if (all.getFirst()) {
+		if (all.getFirst()) 
+		{
+			int cnt = 0;
 			do {
+				if ((cnt++ % 100) == 0)
+				{
+					Database::Checkout cout(database);
+					THREAD_YIELD();
+				}
+
 				WIN window2(DB_PAGE_SPACE, all.current().db_page);
 				NBAK_TRACE(("Merge page %d, diff=%d", all.current().db_page, all.current().diff_page));
 				Ods::pag* page = CCH_FETCH(tdbb, &window2, LCK_write, pag_undefined);
@@ -491,11 +565,15 @@ void BackupManager::endBackup(thread_db* tdbb, bool recover)
 
 		// Page allocation table cache is no longer valid
 		NBAK_TRACE(("Dropping alloc table"));
-		delete alloc_table;
-		alloc_table = NULL;
-		last_allocated_page = 0;
-		if (!allocLock->tryReleaseLock(tdbb))
-			ERR_bugcheck_msg("There are holders of alloc_lock after end_backup finish");
+		{
+			LocalAllocWriteGuard localAllocGuard(this);
+
+			delete alloc_table;
+			alloc_table = NULL;
+			last_allocated_page = 0;
+			if (!allocLock->tryReleaseLock(tdbb))
+				ERR_bugcheck_msg("There are holders of alloc_lock after end_backup finish");
+		}
 
 		closeDelta();
 		unlink(diff_name.c_str());
@@ -511,8 +589,22 @@ void BackupManager::endBackup(thread_db* tdbb, bool recover)
 	return;
 }
 
-bool BackupManager::actualizeAlloc(thread_db* tdbb)
+void BackupManager::initializeAlloc(thread_db* tdbb)
 {
+	StateReadGuard stateGuard(tdbb);
+	if (getState() != nbak_state_normal)
+		actualizeAlloc(tdbb, false);
+}
+
+bool BackupManager::actualizeAlloc(thread_db* tdbb, bool haveGlobalLock)
+{
+	// localAllocLock must be already locked for write here !
+
+	// Difference file pointer pages have one ULONG as number of pages allocated on the page and
+	// then go physical numbers of pages from main database file. Offsets of numbers correspond
+	// to difference file pages.
+	const size_t PAGES_PER_ALLOC_PAGE = database->dbb_page_size / sizeof(ULONG) - 1;
+
 	ISC_STATUS *status_vector = tdbb->tdbb_status_vector;
 	try {
 		NBAK_TRACE(("actualize_alloc last_allocated_page=%d alloc_table=%p",
@@ -521,21 +613,26 @@ bool BackupManager::actualizeAlloc(thread_db* tdbb)
 		// it has exlock or when exclusive access to database is enabled
 		if (!alloc_table)
 			alloc_table = FB_NEW(*database->dbb_permanent) AllocItemTree(database->dbb_permanent);
+
 		while (true)
 		{
 			BufferDesc temp_bdb;
-			// Difference file pointer pages have one ULONG as number of pages allocated on the page and
-			// then go physical numbers of pages from main database file. Offsets of numbers correspond
-			// to difference file pages.
 
 			// Get offset of pointer page. We can do so because page sizes are powers of 2
-			temp_bdb.bdb_page = last_allocated_page & ~(database->dbb_page_size / sizeof(ULONG) - 1);
+			temp_bdb.bdb_page = last_allocated_page & ~PAGES_PER_ALLOC_PAGE;
 			temp_bdb.bdb_dbb = database;
 			temp_bdb.bdb_buffer = reinterpret_cast<Ods::pag*>(alloc_buffer);
 
 			if (!PIO_read(diff_file, &temp_bdb, temp_bdb.bdb_buffer, status_vector)) {
 				return false;
 			}
+
+			// If we have not acquired global allocLock, don't read last page of allocation
+			// table as it could be modified concurrently. Lets read it later when global
+			// alloc lock will be acquired.
+			if (!haveGlobalLock && (alloc_buffer[0] != PAGES_PER_ALLOC_PAGE))
+				break;
+
 			for (ULONG i = last_allocated_page - temp_bdb.bdb_page.getPageNum(); i < alloc_buffer[0]; i++)
 			{
 				NBAK_TRACE(("alloc item page=%d, diff=%d", alloc_buffer[i + 1], temp_bdb.bdb_page.getPageNum() + i + 1));
@@ -548,7 +645,7 @@ bool BackupManager::actualizeAlloc(thread_db* tdbb)
 				}
 			}
 			last_allocated_page = temp_bdb.bdb_page.getPageNum() + alloc_buffer[0];
-			if (alloc_buffer[0] == database->dbb_page_size / sizeof(ULONG) - 1)
+			if (alloc_buffer[0] == PAGES_PER_ALLOC_PAGE)
 				last_allocated_page++;	// if page is full adjust position for next pointer page
 			else
 				break;	// We finished reading allocation table
@@ -562,26 +659,60 @@ bool BackupManager::actualizeAlloc(thread_db* tdbb)
 		last_allocated_page = 0;
 		return false;
 	}
+
+	allocIsValid = haveGlobalLock;
 	return true;
 }
 
+// Return page index in difference file that can be used in
+// writeDifference call later.
+ULONG BackupManager::findPageIndex(thread_db* tdbb, ULONG db_page)
+{
+	// localAllocLock must be already locked here
+
+	NBAK_TRACE(("find_page_index"));
+	if (!alloc_table)
+		return 0;
+
+	AllocItemTree::Accessor a(alloc_table);
+	ULONG diff_page = a.locate(db_page) ? a.current().diff_page : 0;
+
+	return diff_page;
+}
+
 // Return page index in difference file that can be used in
 // writeDifference call later.
 ULONG BackupManager::getPageIndex(thread_db* tdbb, ULONG db_page)
 {
 	NBAK_TRACE(("get_page_index"));
 
-	AllocItemTree::Accessor a(alloc_table);
-	ULONG diff_page = a.locate(db_page) ? a.current().diff_page : 0;
+	{
+		LocalAllocReadGuard localAllocGuard(this);
 
-	return diff_page;
+		const ULONG diff_page = findPageIndex(tdbb, db_page);
+		if (diff_page || backup_state == nbak_state_merge && allocIsValid)
+			return diff_page;
+	}
+
+	LocalAllocWriteGuard localAllocGuard(this);
+	GlobalAllocReadGuard globalAllocGuard(tdbb, this);
+	return findPageIndex(tdbb, db_page);
 }
 
 // Mark next difference page as used by some database page
 ULONG BackupManager::allocateDifferencePage(thread_db* tdbb, ULONG db_page)
 {
-	// This page may be allocated by other
-	if (ULONG diff_page = getPageIndex(tdbb, db_page)) {
+	LocalAllocWriteGuard localAllocGuard(this);
+
+	// This page may be allocated while we wait for a local lock above 
+	if (ULONG diff_page = findPageIndex(tdbb, db_page)) {
+		return diff_page;
+	}
+
+	GlobalAllocWriteGuard globalAllocGuard(tdbb, this);
+
+	// This page may be allocated by other process
+	if (ULONG diff_page = findPageIndex(tdbb, db_page)) {
 		return diff_page;
 	}
 
@@ -682,10 +813,17 @@ void BackupManager::flushDifference()
 	PIO_flush(database, diff_file);
 }
 
+void BackupManager::setForcedWrites(const bool forceWrite, const bool notUseFSCache)
+{
+	if (diff_file) {
+		PIO_force_write(diff_file, forceWrite, notUseFSCache);
+	}
+}
+
 BackupManager::BackupManager(thread_db* tdbb, Database* _database, int ini_state) :
 	dbCreating(false), database(_database), diff_file(NULL), alloc_table(NULL),
 	last_allocated_page(0), current_scn(0), diff_name(*_database->dbb_permanent),
-	explicit_diff_name(false), flushInProgress(false), shuttedDown(false), 
+	explicit_diff_name(false), flushInProgress(false), shuttedDown(false), allocIsValid(false), 
 	stateLock(FB_NEW(*database->dbb_permanent) NBackupStateLock(tdbb, *database->dbb_permanent, this)),
 	allocLock(FB_NEW(*database->dbb_permanent) NBackupAllocLock(tdbb, *database->dbb_permanent, this))
 {
@@ -804,6 +942,8 @@ bool BackupManager::actualizeState(thread_db* tdbb)
 	if (new_backup_state == nbak_state_normal || missed_cycle)
 	{
 		// Page allocation table cache is no longer valid.
+		LocalAllocWriteGuard localAllocGuard(this);
+
 		if (alloc_table) {
 			NBAK_TRACE(("Dropping alloc table"));
 			delete alloc_table;
@@ -812,6 +952,8 @@ bool BackupManager::actualizeState(thread_db* tdbb)
 			if (!allocLock->tryReleaseLock(tdbb))
 				ERR_bugcheck_msg("There are holders of alloc_lock after end_backup finish");
 		}
+
+		closeDelta();
 	}
 
 	if (new_backup_state != nbak_state_normal && !diff_file)
