@@ -198,8 +198,7 @@ jrd_nod* PAR_blr(thread_db* tdbb, jrd_rel* relation, const UCHAR* blr, ULONG blr
 				   Arg::Gds(isc_wroblrver) << Arg::Num(blr_version4) << Arg::Num(version));
 	}
 
-	jrd_nod* node = PAR_parse_node(tdbb, csb, OTHER);
-	csb->csb_node = node;
+	csb->csb_node = PAR_parse_node(tdbb, csb, OTHER);
 
 	if (csb->csb_blr_reader.getByte() != (UCHAR) blr_eoc)
 		PAR_syntax_error(csb, "end_of_command");
@@ -207,7 +206,7 @@ jrd_nod* PAR_blr(thread_db* tdbb, jrd_rel* relation, const UCHAR* blr, ULONG blr
 	if (request_ptr)
 		*request_ptr = CMP_make_request(tdbb, csb, true);
 
-	return node;
+	return csb->csb_node;
 }
 
 
@@ -377,7 +376,7 @@ USHORT PAR_desc(thread_db* tdbb, CompilerScratch* csb, DSC* desc, ItemInfo* item
 
 			FieldInfo fieldInfo;
 			bool exist = csb->csb_map_field_info.get(namePair, fieldInfo);
-			MET_get_domain(tdbb, *name, desc, (exist ? NULL : &fieldInfo));
+			MET_get_domain(tdbb, csb->csb_pool, *name, desc, (exist ? NULL : &fieldInfo));
 
 			if (!exist)
 				csb->csb_map_field_info.put(namePair, fieldInfo);
@@ -440,7 +439,8 @@ USHORT PAR_desc(thread_db* tdbb, CompilerScratch* csb, DSC* desc, ItemInfo* item
 
 			FieldInfo fieldInfo;
 			bool exist = csb->csb_map_field_info.get(namePair, fieldInfo);
-			MET_get_relation_field(tdbb, *relationName, *fieldName, desc, (exist ? NULL : &fieldInfo));
+			MET_get_relation_field(tdbb, csb->csb_pool, *relationName, *fieldName, desc,
+				(exist ? NULL : &fieldInfo));
 
 			if (!exist)
 				csb->csb_map_field_info.put(namePair, fieldInfo);
@@ -1289,7 +1289,7 @@ static jrd_nod* par_field(thread_db* tdbb, CompilerScratch* csb, SSHORT blr_oper
 		node->nod_count = 0;
 
 		dsc* desc = (dsc*) (node->nod_arg + e_domval_desc);
-		MET_get_domain(tdbb, csb->csb_domain_validation, desc, NULL);
+		MET_get_domain(tdbb, csb->csb_pool, csb->csb_domain_validation, desc, NULL);
 
 		return node;
 	}
@@ -2617,7 +2617,7 @@ jrd_nod* PAR_parse_node(thread_db* tdbb, CompilerScratch* csb, USHORT expected)
  **************************************/
 	SET_TDBB(tdbb);
 
-	const USHORT blr_offset = csb->csb_blr_reader.getOffset();
+	const ULONG blr_offset = csb->csb_blr_reader.getOffset();
 	const SSHORT blr_operator = csb->csb_blr_reader.getByte();
 
 	if (blr_operator < 0 || blr_operator >= FB_NELEM(type_table)) {
@@ -3418,7 +3418,7 @@ jrd_nod* PAR_parse_node(thread_db* tdbb, CompilerScratch* csb, USHORT expected)
 	}
 
 	size_t pos = 0;
-	if (csb->csb_dbg_info.blrToSrc.find(blr_offset, pos))
+	if (blr_offset <= MAX_USHORT && csb->csb_dbg_info.blrToSrc.find(blr_offset, pos))
 	{
 		Firebird::MapBlrToSrcItem& i = csb->csb_dbg_info.blrToSrc[pos];
 		jrd_nod* node_src = PAR_make_node(tdbb, e_src_info_length);
