@@ -227,7 +227,10 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
     MWAW_DEBUG_MSG(("SubDocument::parse: no listener\n"));
     return;
   }
-  assert(m_textParser);
+  if (!m_textParser) {
+    MWAW_DEBUG_MSG(("SubDocument::parse: no text parser\n"));
+    return;
+  }
 
   long pos = m_input->tell();
   if (m_type == libmwaw::DOC_HEADER_FOOTER)
@@ -675,7 +678,7 @@ bool DocMkrText::readStyles(MWAWEntry const &entry)
     font.setColor(MWAWColor(col[0],col[1],col[2]));
     font.m_extra=f.str();
     if (zone.m_posFontMap.find(cPos) != zone.m_posFontMap.end()) {
-      MWAW_DEBUG_MSG(("DocMkrText::readStyles: a style for pos=%lx already exist\n", cPos));
+      MWAW_DEBUG_MSG(("DocMkrText::readStyles: a style for pos=%lx already exist\n", (long unsigned int) cPos));
     }
     else
       zone.m_posFontMap[cPos] = font;
@@ -731,7 +734,7 @@ bool DocMkrText::sendTOC()
       prevId=zId;
       listener->setFont(cFont);
 
-      listener->insertUnicodeString(ss.str().c_str());
+      listener->insertUnicodeString(librevenge::RVNGString(ss.str().c_str()));
       listener->insertChar(' ');
       if (m_state->m_idZoneMap.find(127+zId)!=m_state->m_idZoneMap.end())
         sendString(m_state->m_idZoneMap.find(127+zId)->second.m_name);
@@ -740,7 +743,7 @@ bool DocMkrText::sendTOC()
     }
     sendString(toc.m_textList[i]);
     listener->insertTab();
-    listener->insertUnicodeString(ss.str().c_str());
+    listener->insertUnicodeString(librevenge::RVNGString(ss.str().c_str()));
     listener->insertEOL();
   }
   return true;
@@ -921,7 +924,7 @@ bool DocMkrText::sendFooter(int zId)
         break;
       }
       case 5:
-        listener->insertUnicodeString("Page ");
+        listener->insertUnicodeString(librevenge::RVNGString("Page "));
         listener->insertField(MWAWField(MWAWField::PageNumber));
         break;
       case 6:
