@@ -1,22 +1,15 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* libe-book
- * Version: MPL 2.0 / LGPLv2.1+
+/*
+ * This file is part of the libe-book project.
  *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
- *
- * Alternatively, the contents of this file may be used under the terms
- * of the GNU Lesser General Public License Version 2.1 or later
- * (LGPLv2.1+), in which case the provisions of the LGPLv2.1+ are
- * applicable instead of those above.
- *
- * For further information visit http://libebook.sourceforge.net
  */
 
 #include <deque>
 
-#include <libwpd/WPXString.h>
+#include <librevenge/librevenge.h>
 
 #include "FB2Collector.h"
 #include "FB2MetadataContext.h"
@@ -32,18 +25,18 @@ public:
   {
     Data();
 
-    WPXString firstName;
-    WPXString middleName;
-    WPXString lastName;
-    WPXString nickname;
+    librevenge::RVNGString firstName;
+    librevenge::RVNGString middleName;
+    librevenge::RVNGString lastName;
+    librevenge::RVNGString nickname;
   };
 
   FB2Authors();
 
-  void setFirstName(const WPXString &name);
-  void setMiddleName(const WPXString &name);
-  void setLastName(const WPXString &name);
-  void setNickname(const WPXString &name);
+  void setFirstName(const librevenge::RVNGString &name);
+  void setMiddleName(const librevenge::RVNGString &name);
+  void setLastName(const librevenge::RVNGString &name);
+  void setNickname(const librevenge::RVNGString &name);
   void finishAuthor();
 
   const std::deque<Data> &getAuthors() const;
@@ -72,22 +65,22 @@ FB2Authors::FB2Authors()
 {
 }
 
-void FB2Authors::setFirstName(const WPXString &name)
+void FB2Authors::setFirstName(const librevenge::RVNGString &name)
 {
   m_current.firstName = name;
 }
 
-void FB2Authors::setMiddleName(const WPXString &name)
+void FB2Authors::setMiddleName(const librevenge::RVNGString &name)
 {
   m_current.middleName = name;
 }
 
-void FB2Authors::setLastName(const WPXString &name)
+void FB2Authors::setLastName(const librevenge::RVNGString &name)
 {
   m_current.lastName = name;
 }
 
-void FB2Authors::setNickname(const WPXString &name)
+void FB2Authors::setNickname(const librevenge::RVNGString &name)
 {
   m_current.nickname = name;
 }
@@ -113,7 +106,7 @@ FB2AnnotationContext::FB2AnnotationContext(FB2ParserContext *parentContext, cons
 {
 }
 
-FB2XMLParserContext *FB2AnnotationContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2AnnotationContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -138,7 +131,7 @@ void FB2AnnotationContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2AnnotationContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2AnnotationContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -162,7 +155,7 @@ FB2AuthorContext::FB2AuthorContext(FB2ParserContext *parentContext, FB2Authors &
 {
 }
 
-FB2XMLParserContext *FB2AuthorContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2AuthorContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
   {
@@ -197,7 +190,7 @@ void FB2AuthorContext::endOfElement()
   m_authors.finishAuthor();
 }
 
-void FB2AuthorContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2AuthorContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
@@ -210,7 +203,7 @@ FB2BookNameContext::FB2BookNameContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2BookNameContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2BookNameContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -235,7 +228,7 @@ void FB2BookNameContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2BookNameContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2BookNameContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -252,7 +245,7 @@ void FB2BookNameContext::attribute(const EBOOKToken &name, const EBOOKToken *ns,
 void FB2BookNameContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -261,7 +254,7 @@ FB2BookTitleContext::FB2BookTitleContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2BookTitleContext::element(const EBOOKToken &, const EBOOKToken &)
+FB2XMLParserContext *FB2BookTitleContext::element(const FB2TokenData &, const FB2TokenData &)
 {
   return 0;
 }
@@ -274,7 +267,7 @@ void FB2BookTitleContext::endOfElement()
 {
 }
 
-void FB2BookTitleContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2BookTitleContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
@@ -288,7 +281,7 @@ FB2CityContext::FB2CityContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2CityContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2CityContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -313,7 +306,7 @@ void FB2CityContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2CityContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2CityContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -330,7 +323,7 @@ void FB2CityContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, con
 void FB2CityContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -339,7 +332,7 @@ FB2CoverpageContext::FB2CoverpageContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2CoverpageContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2CoverpageContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -364,7 +357,7 @@ void FB2CoverpageContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2CoverpageContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2CoverpageContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -381,7 +374,7 @@ void FB2CoverpageContext::attribute(const EBOOKToken &name, const EBOOKToken *ns
 void FB2CoverpageContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -392,7 +385,7 @@ FB2CustomInfoContext::FB2CustomInfoContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2CustomInfoContext::element(const EBOOKToken &, const EBOOKToken &)
+FB2XMLParserContext *FB2CustomInfoContext::element(const FB2TokenData &, const FB2TokenData &)
 {
   return 0;
 }
@@ -403,14 +396,14 @@ void FB2CustomInfoContext::startOfElement()
 
 void FB2CustomInfoContext::endOfElement()
 {
-  WPXString info(m_type);
+  librevenge::RVNGString info(m_type);
   info.append(": ");
   info.append(m_text);
   // TODO: there might be more than one comment. Must properties be unique?
-  getCollector()->defineMetadataEntry("libwpd:comments", info.cstr());
+  getCollector()->defineMetadataEntry("librevenge:comments", info.cstr());
 }
 
-void FB2CustomInfoContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2CustomInfoContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   if (FB2_NO_NAMESPACE(ns))
   {
@@ -435,7 +428,7 @@ FB2DateContext::FB2DateContext(FB2ParserContext *const parentContext, const char
 {
 }
 
-FB2XMLParserContext *FB2DateContext::element(const EBOOKToken &, const EBOOKToken &)
+FB2XMLParserContext *FB2DateContext::element(const FB2TokenData &, const FB2TokenData &)
 {
   return 0;
 }
@@ -448,7 +441,7 @@ void FB2DateContext::endOfElement()
 {
 }
 
-void FB2DateContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2DateContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: should we use the computer-readable date from attribute, if
   // present?
@@ -475,7 +468,7 @@ FB2DescriptionContext::FB2DescriptionContext(FB2ParserContext *const parentConte
 {
 }
 
-FB2XMLParserContext *FB2DescriptionContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2DescriptionContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
   {
@@ -487,7 +480,7 @@ FB2XMLParserContext *FB2DescriptionContext::element(const EBOOKToken &name, cons
       // not interesting -> skip
       break;
     case FB2Token::document_info :
-      return new DocumentInfoContext(this);
+      return new TextInfoContext(this);
     case FB2Token::publish_info :
       return new FB2PublishInfoContext(this);
     case FB2Token::custom_info :
@@ -510,7 +503,7 @@ void FB2DescriptionContext::endOfElement()
 {
 }
 
-void FB2DescriptionContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2DescriptionContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
@@ -518,18 +511,18 @@ void FB2DescriptionContext::text(const char *)
 {
 }
 
-DocumentInfoContext::DocumentInfoContext(FB2ParserContext *parentContext)
+TextInfoContext::TextInfoContext(FB2ParserContext *parentContext)
   : FB2ParserContext(parentContext)
   , m_authors(new FB2Authors())
 {
 }
 
-DocumentInfoContext::~DocumentInfoContext()
+TextInfoContext::~TextInfoContext()
 {
   delete m_authors;
 }
 
-FB2XMLParserContext *DocumentInfoContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *TextInfoContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
   {
@@ -556,20 +549,20 @@ FB2XMLParserContext *DocumentInfoContext::element(const EBOOKToken &name, const
   return new FB2SkipElementContext(this);
 }
 
-void DocumentInfoContext::startOfElement()
+void TextInfoContext::startOfElement()
 {
 }
 
-void DocumentInfoContext::endOfElement()
+void TextInfoContext::endOfElement()
 {
   // TODO: handle authors
 }
 
-void DocumentInfoContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void TextInfoContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
-void DocumentInfoContext::text(const char *)
+void TextInfoContext::text(const char *)
 {
 }
 
@@ -578,7 +571,7 @@ FB2EmailContext::FB2EmailContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2EmailContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2EmailContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -603,7 +596,7 @@ void FB2EmailContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2EmailContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2EmailContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -620,7 +613,7 @@ void FB2EmailContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, co
 void FB2EmailContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -630,7 +623,7 @@ FB2FirstNameContext::FB2FirstNameContext(FB2ParserContext *parentContext, FB2Aut
 {
 }
 
-FB2XMLParserContext *FB2FirstNameContext::element(const EBOOKToken &, const EBOOKToken &)
+FB2XMLParserContext *FB2FirstNameContext::element(const FB2TokenData &, const FB2TokenData &)
 {
   return 0;
 }
@@ -643,13 +636,13 @@ void FB2FirstNameContext::endOfElement()
 {
 }
 
-void FB2FirstNameContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2FirstNameContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
 void FB2FirstNameContext::text(const char *value)
 {
-  const WPXString str(value);
+  const librevenge::RVNGString str(value);
   m_authors.setFirstName(str);
 }
 
@@ -658,7 +651,7 @@ FB2GenreContext::FB2GenreContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2GenreContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2GenreContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -683,7 +676,7 @@ void FB2GenreContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2GenreContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2GenreContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -700,7 +693,7 @@ void FB2GenreContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, co
 void FB2GenreContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -709,7 +702,7 @@ FB2HistoryContext::FB2HistoryContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2HistoryContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2HistoryContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -734,7 +727,7 @@ void FB2HistoryContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2HistoryContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2HistoryContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -751,7 +744,7 @@ void FB2HistoryContext::attribute(const EBOOKToken &name, const EBOOKToken *ns,
 void FB2HistoryContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -760,7 +753,7 @@ FB2HomePageContext::FB2HomePageContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2HomePageContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2HomePageContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -785,7 +778,7 @@ void FB2HomePageContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2HomePageContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2HomePageContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -802,7 +795,7 @@ void FB2HomePageContext::attribute(const EBOOKToken &name, const EBOOKToken *ns,
 void FB2HomePageContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -811,7 +804,7 @@ FB2IdContext::FB2IdContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2IdContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2IdContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -836,7 +829,7 @@ void FB2IdContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2IdContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2IdContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -853,7 +846,7 @@ void FB2IdContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const
 void FB2IdContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -862,7 +855,7 @@ FB2IsbnContext::FB2IsbnContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2IsbnContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2IsbnContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -887,7 +880,7 @@ void FB2IsbnContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2IsbnContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2IsbnContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -904,7 +897,7 @@ void FB2IsbnContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, con
 void FB2IsbnContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -913,7 +906,7 @@ FB2KeywordsContext::FB2KeywordsContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2KeywordsContext::element(const EBOOKToken &, const EBOOKToken &)
+FB2XMLParserContext *FB2KeywordsContext::element(const FB2TokenData &, const FB2TokenData &)
 {
   return 0;
 }
@@ -926,7 +919,7 @@ void FB2KeywordsContext::endOfElement()
 {
 }
 
-void FB2KeywordsContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2KeywordsContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
@@ -940,7 +933,7 @@ FB2LangContext::FB2LangContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2LangContext::element(const EBOOKToken &, const EBOOKToken &)
+FB2XMLParserContext *FB2LangContext::element(const FB2TokenData &, const FB2TokenData &)
 {
   return 0;
 }
@@ -953,7 +946,7 @@ void FB2LangContext::endOfElement()
 {
 }
 
-void FB2LangContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2LangContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
@@ -968,7 +961,7 @@ FB2LastNameContext::FB2LastNameContext(FB2ParserContext *parentContext, FB2Autho
 {
 }
 
-FB2XMLParserContext *FB2LastNameContext::element(const EBOOKToken &, const EBOOKToken &)
+FB2XMLParserContext *FB2LastNameContext::element(const FB2TokenData &, const FB2TokenData &)
 {
   return 0;
 }
@@ -981,13 +974,13 @@ void FB2LastNameContext::endOfElement()
 {
 }
 
-void FB2LastNameContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2LastNameContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
 void FB2LastNameContext::text(const char *value)
 {
-  const WPXString str(value);
+  const librevenge::RVNGString str(value);
   m_authors.setLastName(str);
 }
 
@@ -997,7 +990,7 @@ FB2MiddleNameContext::FB2MiddleNameContext(FB2ParserContext *parentContext, FB2A
 {
 }
 
-FB2XMLParserContext *FB2MiddleNameContext::element(const EBOOKToken &, const EBOOKToken &)
+FB2XMLParserContext *FB2MiddleNameContext::element(const FB2TokenData &, const FB2TokenData &)
 {
   return 0;
 }
@@ -1010,13 +1003,13 @@ void FB2MiddleNameContext::endOfElement()
 {
 }
 
-void FB2MiddleNameContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2MiddleNameContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
 void FB2MiddleNameContext::text(const char *value)
 {
-  const WPXString str(value);
+  const librevenge::RVNGString str(value);
   m_authors.setMiddleName(str);
 }
 
@@ -1026,7 +1019,7 @@ FB2NicknameContext::FB2NicknameContext(FB2ParserContext *parentContext, FB2Autho
 {
 }
 
-FB2XMLParserContext *FB2NicknameContext::element(const EBOOKToken &, const EBOOKToken &)
+FB2XMLParserContext *FB2NicknameContext::element(const FB2TokenData &, const FB2TokenData &)
 {
   return 0;
 }
@@ -1039,13 +1032,13 @@ void FB2NicknameContext::endOfElement()
 {
 }
 
-void FB2NicknameContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2NicknameContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
 void FB2NicknameContext::text(const char *value)
 {
-  const WPXString str(value);
+  const librevenge::RVNGString str(value);
   m_authors.setNickname(str);
 }
 
@@ -1054,7 +1047,7 @@ FB2OutputContext::FB2OutputContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2OutputContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2OutputContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -1079,7 +1072,7 @@ void FB2OutputContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2OutputContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2OutputContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -1096,7 +1089,7 @@ void FB2OutputContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, c
 void FB2OutputContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -1105,7 +1098,7 @@ FB2OutputDocumentClassContext::FB2OutputDocumentClassContext(FB2ParserContext *p
 {
 }
 
-FB2XMLParserContext *FB2OutputDocumentClassContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2OutputDocumentClassContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -1130,7 +1123,7 @@ void FB2OutputDocumentClassContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2OutputDocumentClassContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2OutputDocumentClassContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -1147,7 +1140,7 @@ void FB2OutputDocumentClassContext::attribute(const EBOOKToken &name, const EBOO
 void FB2OutputDocumentClassContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -1156,7 +1149,7 @@ FB2PartContext::FB2PartContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2PartContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2PartContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -1181,7 +1174,7 @@ void FB2PartContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2PartContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2PartContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -1198,7 +1191,7 @@ void FB2PartContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, con
 void FB2PartContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -1207,7 +1200,7 @@ FB2ProgramUsedContext::FB2ProgramUsedContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2ProgramUsedContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2ProgramUsedContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -1232,7 +1225,7 @@ void FB2ProgramUsedContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2ProgramUsedContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2ProgramUsedContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -1249,7 +1242,7 @@ void FB2ProgramUsedContext::attribute(const EBOOKToken &name, const EBOOKToken *
 void FB2ProgramUsedContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -1258,7 +1251,7 @@ FB2PublisherContext::FB2PublisherContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2PublisherContext::element(const EBOOKToken &, const EBOOKToken &)
+FB2XMLParserContext *FB2PublisherContext::element(const FB2TokenData &, const FB2TokenData &)
 {
   return 0;
 }
@@ -1271,7 +1264,7 @@ void FB2PublisherContext::endOfElement()
 {
 }
 
-void FB2PublisherContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2PublisherContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
@@ -1285,7 +1278,7 @@ FB2PublishInfoContext::FB2PublishInfoContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2PublishInfoContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2PublishInfoContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
   {
@@ -1316,7 +1309,7 @@ void FB2PublishInfoContext::endOfElement()
 {
 }
 
-void FB2PublishInfoContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2PublishInfoContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
@@ -1329,7 +1322,7 @@ FB2SequenceContext::FB2SequenceContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2SequenceContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2SequenceContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -1354,7 +1347,7 @@ void FB2SequenceContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2SequenceContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2SequenceContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -1371,7 +1364,7 @@ void FB2SequenceContext::attribute(const EBOOKToken &name, const EBOOKToken *ns,
 void FB2SequenceContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -1380,7 +1373,7 @@ FB2SrcLangContext::FB2SrcLangContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2SrcLangContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2SrcLangContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -1405,7 +1398,7 @@ void FB2SrcLangContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2SrcLangContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2SrcLangContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -1422,7 +1415,7 @@ void FB2SrcLangContext::attribute(const EBOOKToken &name, const EBOOKToken *ns,
 void FB2SrcLangContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -1431,7 +1424,7 @@ FB2SrcOcrContext::FB2SrcOcrContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2SrcOcrContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2SrcOcrContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -1456,7 +1449,7 @@ void FB2SrcOcrContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2SrcOcrContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2SrcOcrContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -1473,7 +1466,7 @@ void FB2SrcOcrContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, c
 void FB2SrcOcrContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -1482,7 +1475,7 @@ FB2SrcTitleInfoContext::FB2SrcTitleInfoContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2SrcTitleInfoContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2SrcTitleInfoContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -1507,7 +1500,7 @@ void FB2SrcTitleInfoContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2SrcTitleInfoContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2SrcTitleInfoContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -1524,7 +1517,7 @@ void FB2SrcTitleInfoContext::attribute(const EBOOKToken &name, const EBOOKToken
 void FB2SrcTitleInfoContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -1533,7 +1526,7 @@ FB2SrcUrlContext::FB2SrcUrlContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2SrcUrlContext::element(const EBOOKToken &, const EBOOKToken &)
+FB2XMLParserContext *FB2SrcUrlContext::element(const FB2TokenData &, const FB2TokenData &)
 {
   return 0;
 }
@@ -1546,7 +1539,7 @@ void FB2SrcUrlContext::endOfElement()
 {
 }
 
-void FB2SrcUrlContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2SrcUrlContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
@@ -1566,7 +1559,7 @@ FB2TitleInfoContext::~FB2TitleInfoContext()
   delete m_authors;
 }
 
-FB2XMLParserContext *FB2TitleInfoContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2TitleInfoContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -1608,10 +1601,10 @@ void FB2TitleInfoContext::endOfElement()
 {
   typedef std::deque<FB2Authors::Data> Authors_t;
   const Authors_t authors = m_authors->getAuthors();
-  WPXString authorsStr;
+  librevenge::RVNGString authorsStr;
   for (Authors_t::const_iterator it = authors.begin(); it != authors.end(); ++it)
   {
-    WPXString author;
+    librevenge::RVNGString author;
     if (it->firstName.len() != 0)
     {
       author = it->firstName;
@@ -1642,7 +1635,7 @@ void FB2TitleInfoContext::endOfElement()
     getCollector()->defineMetadataEntry("meta:initial-creator", authorsStr.cstr());
 }
 
-void FB2TitleInfoContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2TitleInfoContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
@@ -1655,7 +1648,7 @@ FB2TranslatorContext::FB2TranslatorContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2TranslatorContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2TranslatorContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -1680,7 +1673,7 @@ void FB2TranslatorContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2TranslatorContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2TranslatorContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -1697,7 +1690,7 @@ void FB2TranslatorContext::attribute(const EBOOKToken &name, const EBOOKToken *n
 void FB2TranslatorContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
@@ -1706,7 +1699,7 @@ FB2VersionContext::FB2VersionContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2VersionContext::element(const EBOOKToken &, const EBOOKToken &)
+FB2XMLParserContext *FB2VersionContext::element(const FB2TokenData &, const FB2TokenData &)
 {
   return 0;
 }
@@ -1719,13 +1712,13 @@ void FB2VersionContext::endOfElement()
 {
 }
 
-void FB2VersionContext::attribute(const EBOOKToken &, const EBOOKToken *, const char *)
+void FB2VersionContext::attribute(const FB2TokenData &, const FB2TokenData *, const char *)
 {
 }
 
 void FB2VersionContext::text(const char *value)
 {
-  getCollector()->defineMetadataEntry("libwpd:version-number", value);
+  getCollector()->defineMetadataEntry("librevenge:version-number", value);
 }
 
 FB2YearContext::FB2YearContext(FB2ParserContext *parentContext)
@@ -1733,7 +1726,7 @@ FB2YearContext::FB2YearContext(FB2ParserContext *parentContext)
 {
 }
 
-FB2XMLParserContext *FB2YearContext::element(const EBOOKToken &name, const EBOOKToken &ns)
+FB2XMLParserContext *FB2YearContext::element(const FB2TokenData &name, const FB2TokenData &ns)
 {
   // TODO: implement me
   if (FB2Token::NS_FICTIONBOOK == getFB2TokenID(ns))
@@ -1758,7 +1751,7 @@ void FB2YearContext::endOfElement()
   // TODO: implement me
 }
 
-void FB2YearContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, const char *value)
+void FB2YearContext::attribute(const FB2TokenData &name, const FB2TokenData *ns, const char *value)
 {
   // TODO: implement me
   (void) value;
@@ -1775,7 +1768,7 @@ void FB2YearContext::attribute(const EBOOKToken &name, const EBOOKToken *ns, con
 void FB2YearContext::text(const char *value)
 {
   // TODO: implement me
-  WPXString str(value);
+  librevenge::RVNGString str(value);
   (void) str;
 }
 
