@@ -12,7 +12,6 @@
 
 #include <boost/static_assert.hpp>
 
-#include "IWORKToken.h"
 #include "IWORKTokenInfo.h"
 
 namespace libetonyek
@@ -47,9 +46,11 @@ enum
   size,
   slide,
   slide_list,
+  slide_style,
   sticky_note,
   sticky_notes,
   string,
+  style_ref,
   stylesheet,
   text,
   theme,
