@@ -262,8 +262,8 @@ MWAWTextListenerPtr MsWks4Zone::createListener(librevenge::RVNGTextInterface *in
   pagesH.resize(size_t(m_state->m_numPages), int(72.*getTextHeight()));
   m_document->getGraphParser()->computePositions(-1, linesH, pagesH);
   m_document->getGraphParser()->setPageLeftTop
-  (Vec2f(72.f*float(m_parserState->m_pageSpan.getMarginLeft()),
-         72.f*float(m_parserState->m_pageSpan.getMarginTop())+float(m_state->m_headerHeight)));
+  (MWAWVec2f(72.f*float(m_parserState->m_pageSpan.getMarginLeft()),
+             72.f*float(m_parserState->m_pageSpan.getMarginTop())+float(m_state->m_headerHeight)));
 
   return res;
 }
@@ -309,7 +309,7 @@ bool MsWks4Zone::parseHeaderIndexEntry(MWAWInputStreamPtr &input)
     if (name[i] != 0 && name[i] != 0x20 &&
         (41 > (uint8_t)name[i] || (uint8_t)name[i] > 90)) {
       MWAW_DEBUG_MSG(("MsWks4Zone:parseHeaderIndexEntry: bad character=%u (0x%02x) in name in header index\n",
-                      (uint8_t)name[i], (uint8_t)name[i]));
+                      (unsigned int)name[i], (unsigned int)name[i]));
       m_document->ascii().addNote("###IndexEntry bad name(ignored)");
 
       input->seek(pos + cch, librevenge::RVNG_SEEK_SET);
@@ -630,9 +630,9 @@ bool MsWks4Zone::readPRNT(MWAWInputStreamPtr input, MWAWEntry const &entry, MWAW
     return false;
   }
   else {
-    Vec2i paperSize = info.paper().size();
-    Vec2i pageSize = info.page().size();
-    Vec2i margin = paperSize - pageSize;
+    MWAWVec2i paperSize = info.paper().size();
+    MWAWVec2i pageSize = info.page().size();
+    MWAWVec2i margin = paperSize - pageSize;
     margin.set(margin.x()/2, margin.y()/2);
 
     page.setMarginTop(margin.y()/72.0);
@@ -699,8 +699,6 @@ bool MsWks4Zone::readDOP(MWAWInputStreamPtr input, MWAWEntry const &entry, MWAWP
       }
       long d = (int) input->readULong(2);
       if (d == 4000) break;
-      int pos = val-0x41;
-      if (pos > 1) pos += 1-2*(pos%2);
       dim[val-0x41] = double(d)/72.;
       break;
     }
@@ -940,13 +938,12 @@ bool MsWks4Zone::readFRAM(MWAWInputStreamPtr input, MWAWEntry const &entry)
     if (endPos > endPage) break;
 
     MsWks4ZoneInternal::Frame frame;
-    Vec2f fOrig, fSiz;
+    MWAWVec2f fOrig, fSiz;
     while (input->tell() < endPos) {
       int val = (int) input->readULong(1);
       long pos = input->tell();
 
       int sz = 0;
-      int szChaine = 0;
       bool done = true;
       switch (val) {
       case 0x2e: // x
@@ -1045,21 +1042,13 @@ bool MsWks4Zone::readFRAM(MWAWInputStreamPtr input, MWAWEntry const &entry)
       if (!ok) break;
       if (done) continue;
 
-      if ((sz == 0 && szChaine == 0) ||
-          (pos + sz + szChaine > endPos)) {
+      if (sz == 0 || pos + sz > endPos) {
         input->seek(-1, librevenge::RVNG_SEEK_CUR);
         ok = false;
         break;
       }
 
       f << std::hex << val << "=" << std::dec;
-
-      if (szChaine) {
-        std::string s;
-        for (int j = 0; j < szChaine; j++)
-          s += (char) input->readULong(1);
-        f << s << ",";
-      }
       if (sz) {
         int v2 = (int) input->readLong(sz);
         f << v2;
