@@ -184,7 +184,7 @@ bool SubDocument::operator!=(MWAWSubDocument const &doc) const
 ////////////////////////////////////////////////////////////
 MsWksDocument::MsWksDocument(MWAWInputStreamPtr input, MWAWParser &parser) :
   m_state(), m_parserState(parser.getParserState()), m_parser(&parser), m_parentDocument(0),
-  m_input(input), m_asciiFile(), m_graphParser(), m_textParser3(), m_textParser4(), m_newPage(0)
+  m_input(input), m_asciiFile(MWAWInputStreamPtr()), m_graphParser(), m_textParser3(), m_textParser4(), m_newPage(0)
 {
   m_state.reset(new MsWksDocumentInternal::State());
   m_graphParser.reset(new MsWksGraph(*this));
@@ -404,7 +404,7 @@ void MsWksDocument::sendOLE(int id, MWAWPosition const &pictPos, MWAWGraphicStyl
     MWAW_DEBUG_MSG(("MsWksDocument::sendOLE: can not find OLE%d\n", id));
     return;
   }
-  m_parserState->getMainListener()->insertPicture(pictPos, data, type, style);
+  m_parserState->getMainListener()->insertPicture(pictPos, MWAWEmbeddedObject(data, type), style);
 }
 
 void MsWksDocument::sendRBIL(int id, MWAWVec2i const &sz)
