@@ -19,14 +19,15 @@
 #include "VSDDocumentStructure.h"
 #include "VSDContentCollector.h"
 #include "VSDStylesCollector.h"
+#include "VSDMetaData.h"
 
-libvisio::VSDParser::VSDParser(librevenge::RVNGInputStream *input, librevenge::RVNGDrawingInterface *painter)
-  : m_input(input), m_painter(painter), m_header(), m_collector(0), m_shapeList(), m_currentLevel(0),
+libvisio::VSDParser::VSDParser(librevenge::RVNGInputStream *input, librevenge::RVNGDrawingInterface *painter, librevenge::RVNGInputStream *container)
+  : m_input(input), m_painter(painter), m_container(container), m_header(), m_collector(0), m_shapeList(), m_currentLevel(0),
     m_stencils(), m_currentStencil(0), m_shape(), m_isStencilStarted(false), m_isInStyles(false),
-    m_currentShapeLevel(0), m_currentShapeID(MINUS_ONE), m_extractStencils(false), m_colours(),
+    m_currentShapeLevel(0), m_currentShapeID(MINUS_ONE), m_currentLayerListLevel(0), m_extractStencils(false), m_colours(),
     m_isBackgroundPage(false), m_isShapeStarted(false), m_shadowOffsetX(0.0), m_shadowOffsetY(0.0),
     m_currentGeometryList(0), m_currentGeomListCount(0), m_fonts(), m_names(), m_namesMapMap(),
-    m_currentPageName()
+    m_currentPageName(), m_currentTabSet()
 {}
 
 libvisio::VSDParser::~VSDParser()
@@ -136,6 +137,9 @@ bool libvisio::VSDParser::parseMain()
 
   VSDContentCollector contentCollector(m_painter, groupXFormsSequence, groupMembershipsSequence, documentPageShapeOrders, styles, m_stencils);
   m_collector = &contentCollector;
+  if (m_container)
+    parseMetaData();
+
   VSD_DEBUG_MSG(("VSDParser::parseMain 2nd pass\n"));
   if (!parseDocument(&trailerStream, shift))
     return false;
@@ -143,6 +147,32 @@ bool libvisio::VSDParser::parseMain()
   return true;
 }
 
+void libvisio::VSDParser::parseMetaData() try
+{
+  if (!m_container)
+    return;
+  m_container->seek(0, librevenge::RVNG_SEEK_SET);
+  if (!m_container->isStructured())
+    return;
+  VSDMetaData metaData;
+
+  const RVNGInputStreamPtr_t sumaryInfo(m_container->getSubStreamByName("\x05SummaryInformation"));
+  if (bool(sumaryInfo))
+    metaData.parse(sumaryInfo.get());
+
+  const RVNGInputStreamPtr_t docSumaryInfo(m_container->getSubStreamByName("\005DocumentSummaryInformation"));
+  if (bool(docSumaryInfo))
+    metaData.parse(docSumaryInfo.get());
+
+  m_container->seek(0, librevenge::RVNG_SEEK_SET);
+  metaData.parseTimes(m_container);
+  m_collector->collectMetaData(metaData.getMetaData());
+}
+catch (...)
+{
+  // Ignore any exceptions in metadata. They are not important enough to stop parsing.
+}
+
 bool libvisio::VSDParser::parseDocument(librevenge::RVNGInputStream *input, unsigned shift)
 {
   try
@@ -201,13 +231,16 @@ void libvisio::VSDParser::handleStreams(librevenge::RVNGInputStream *input, unsi
     {
       Pointer ptr;
       readPointer(input, ptr);
+      if (ptr.Type == 0)
+        continue;
+
       if (ptr.Type == VSD_FONTFACES)
         FontFaces[i] = ptr;
       else if (ptr.Type == VSD_NAME_LIST2)
         NameList[i] = ptr;
       else if (ptr.Type == VSD_NAMEIDX || ptr.Type == VSD_NAMEIDX123)
         NameIDX[i] = ptr;
-      else if (ptr.Type != 0)
+      else if (ptr.Type)
         PtrList[i] = ptr;
     }
     if (listSize <= 1)
@@ -415,6 +448,9 @@ void libvisio::VSDParser::handleChunk(librevenge::RVNGInputStream *input)
   case VSD_XFORM_DATA:
     readXFormData(input);
     break;
+  case VSD_XFORM_1D:
+    readXForm1D(input);
+    break;
   case VSD_TEXT_XFORM:
     readTxtXForm(input);
     break;
@@ -554,6 +590,23 @@ void libvisio::VSDParser::handleChunk(librevenge::RVNGInputStream *input)
   case VSD_MISC:
     readMisc(input);
     break;
+  case VSD_LAYER_LIST:
+    readLayerList(input);
+    break;
+  case VSD_LAYER:
+    readLayer(input);
+    break;
+  case VSD_LAYER_MEMBERSHIP:
+    readLayerMem(input);
+    break;
+  case VSD_TABS_DATA_LIST:
+    readTabsDataList(input);
+    break;
+  case VSD_TABS_DATA_1:
+  case VSD_TABS_DATA_2:
+  case VSD_TABS_DATA_3:
+    readTabsData(input);
+    break;
   default:
     m_collector->collectUnhandledChunk(m_header.id, m_header.level);
   }
@@ -570,13 +623,15 @@ void libvisio::VSDParser::_flushShape()
 
   m_collector->collectXFormData(m_currentShapeLevel+2, m_shape.m_xform);
 
+  m_collector->collectLayerMem(m_currentShapeLevel+2, m_shape.m_layerMem);
+
   m_collector->collectMisc(m_currentShapeLevel+2, m_shape.m_misc);
 
   if (m_shape.m_txtxform)
     m_collector->collectTxtXForm(m_currentShapeLevel+2, *(m_shape.m_txtxform));
 
   m_collector->collectLine(m_currentShapeLevel+2, m_shape.m_lineStyle.width, m_shape.m_lineStyle.colour, m_shape.m_lineStyle.pattern,
-                           m_shape.m_lineStyle.startMarker, m_shape.m_lineStyle.endMarker, m_shape.m_lineStyle.cap);
+                           m_shape.m_lineStyle.startMarker, m_shape.m_lineStyle.endMarker, m_shape.m_lineStyle.cap, m_shape.m_lineStyle.rounding);
 
   m_collector->collectFillAndShadow(m_currentShapeLevel+2, m_shape.m_fillStyle.fgColour, m_shape.m_fillStyle.bgColour, m_shape.m_fillStyle.pattern,
                                     m_shape.m_fillStyle.fgTransparency, m_shape.m_fillStyle.bgTransparency, m_shape.m_fillStyle.shadowPattern,
@@ -605,6 +660,8 @@ void libvisio::VSDParser::_flushShape()
   if (m_shape.m_foreign && m_shape.m_foreign->data.size())
     m_collector->collectForeignData(m_currentShapeLevel+1, m_shape.m_foreign->data);
 
+  m_collector->collectTabsDataList(m_currentShapeLevel+1, m_shape.m_tabSets);
+
   if (!m_shape.m_fields.empty())
     m_shape.m_fields.handle(m_collector);
 
@@ -625,7 +682,9 @@ void libvisio::VSDParser::_flushShape()
 
   m_collector->collectDefaultParaStyle(m_shape.m_paraStyle.charCount, m_shape.m_paraStyle.indFirst, m_shape.m_paraStyle.indLeft,
                                        m_shape.m_paraStyle.indRight, m_shape.m_paraStyle.spLine, m_shape.m_paraStyle.spBefore,
-                                       m_shape.m_paraStyle.spAfter, m_shape.m_paraStyle.align, m_shape.m_paraStyle.flags);
+                                       m_shape.m_paraStyle.spAfter, m_shape.m_paraStyle.align, m_shape.m_paraStyle.bullet,
+                                       m_shape.m_paraStyle.bulletStr, m_shape.m_paraStyle.bulletFont, m_shape.m_paraStyle.bulletFontSize,
+                                       m_shape.m_paraStyle.textPosAfterBullet, m_shape.m_paraStyle.flags);
 
   m_shape.m_paraList.handle(m_collector);
 }
@@ -636,8 +695,11 @@ void libvisio::VSDParser::_handleLevelChange(unsigned level)
     return;
   if (level <= m_currentShapeLevel+1)
   {
-    if (!m_shape.m_geometries.empty() && m_currentGeometryList->empty())
+    if (!m_shape.m_geometries.empty() && m_currentGeometryList && m_currentGeometryList->empty())
+    {
       m_shape.m_geometries.erase(--m_currentGeomListCount);
+      m_currentGeometryList = 0;
+    }
     m_collector->collectShapesOrder(0, m_currentShapeLevel+2, m_shapeList.getShapesOrder());
     m_shapeList.clear();
 
@@ -711,6 +773,20 @@ void libvisio::VSDParser::readOLEData(librevenge::RVNGInputStream *input)
 
 }
 
+void libvisio::VSDParser::readTabsData(librevenge::RVNGInputStream *input)
+{
+  m_shape.m_tabSets[m_header.id].m_numChars = getUInt(input);
+  unsigned char numStops = readU8(input);
+  m_shape.m_tabSets[m_header.id].m_tabStops.clear();
+  for (unsigned char i = 0; i < numStops; ++i)
+  {
+    input->seek(1, librevenge::RVNG_SEEK_CUR);
+    m_shape.m_tabSets[m_header.id].m_tabStops[i].m_position = readDouble(input);
+    m_shape.m_tabSets[m_header.id].m_tabStops[i].m_alignment = readU8(input);
+    m_shape.m_tabSets[m_header.id].m_tabStops[i].m_leader = readU8(input);
+  }
+}
+
 void libvisio::VSDParser::readNameIDX(librevenge::RVNGInputStream *input)
 {
   std::map<unsigned, VSDName> names;
@@ -721,7 +797,6 @@ void libvisio::VSDParser::readNameIDX(librevenge::RVNGInputStream *input)
     if (nameId != readU32(input))
     {
       VSD_DEBUG_MSG(("VSDParser::readNameIDX --> mismatch of first two dwords\n"));
-      return;
     }
     unsigned elementId = readU32(input);
     input->seek(1, librevenge::RVNG_SEEK_CUR);
@@ -778,15 +853,17 @@ void libvisio::VSDParser::readLine(librevenge::RVNGInputStream *input)
   c.b = readU8(input);
   c.a = readU8(input);
   unsigned char linePattern = readU8(input);
-  input->seek(10, librevenge::RVNG_SEEK_CUR);
+  input->seek(1, librevenge::RVNG_SEEK_CUR);
+  double rounding = readDouble(input);
+  input->seek(1, librevenge::RVNG_SEEK_CUR);
   unsigned char startMarker = readU8(input);
   unsigned char endMarker = readU8(input);
   unsigned char lineCap = readU8(input);
 
   if (m_isInStyles)
-    m_collector->collectLineStyle(m_header.level, strokeWidth, c, linePattern, startMarker, endMarker, lineCap);
+    m_collector->collectLineStyle(m_header.level, strokeWidth, c, linePattern, startMarker, endMarker, lineCap, rounding);
   else
-    m_shape.m_lineStyle.override(VSDOptionalLineStyle(strokeWidth, c, linePattern, startMarker, endMarker, lineCap));
+    m_shape.m_lineStyle.override(VSDOptionalLineStyle(strokeWidth, c, linePattern, startMarker, endMarker, lineCap, rounding));
 }
 
 void libvisio::VSDParser::readTextBlock(librevenge::RVNGInputStream *input)
@@ -821,7 +898,7 @@ void libvisio::VSDParser::readTextBlock(librevenge::RVNGInputStream *input)
 
 void libvisio::VSDParser::readGeomList(librevenge::RVNGInputStream *input)
 {
-  if (!m_shape.m_geometries.empty() && m_currentGeometryList->empty())
+  if (!m_shape.m_geometries.empty() && m_currentGeometryList && m_currentGeometryList->empty())
     m_shape.m_geometries.erase(--m_currentGeomListCount);
   // Since this is a map, this will default construct an element and then
   // the m_currentGeometryList pointer takes its address and we will work
@@ -865,7 +942,6 @@ void libvisio::VSDParser::readCharList(librevenge::RVNGInputStream *input)
 
     m_shape.m_charList.setElementsOrder(characterOrder);
   }
-
 }
 
 void libvisio::VSDParser::readParaList(librevenge::RVNGInputStream *input)
@@ -892,6 +968,82 @@ void libvisio::VSDParser::readPropList(librevenge::RVNGInputStream * /* input */
 {
 }
 
+void libvisio::VSDParser::readTabsDataList(librevenge::RVNGInputStream *input)
+{
+  // We want the collectors to still get the level information
+  if (!m_isStencilStarted)
+    m_collector->collectUnhandledChunk(m_header.id, m_header.level);
+
+  if (m_header.trailer)
+  {
+    uint32_t subHeaderLength = readU32(input);
+    uint32_t childrenListLength = readU32(input);
+    input->seek(subHeaderLength, librevenge::RVNG_SEEK_CUR);
+    std::vector<unsigned> tabsOrder;
+    tabsOrder.reserve(childrenListLength / sizeof(uint32_t));
+    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+      tabsOrder.push_back(readU32(input));
+  }
+}
+
+void libvisio::VSDParser::readLayerList(librevenge::RVNGInputStream *input)
+{
+  // We want the collectors to still get the level information
+  if (!m_isStencilStarted)
+    m_collector->collectUnhandledChunk(m_header.id, m_header.level);
+
+  if (m_header.trailer)
+  {
+    uint32_t subHeaderLength = readU32(input);
+    uint32_t childrenListLength = readU32(input);
+    input->seek(subHeaderLength, librevenge::RVNG_SEEK_CUR);
+    std::vector<unsigned> layerOrder;
+    layerOrder.reserve(childrenListLength / sizeof(uint32_t));
+    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+      layerOrder.push_back(readU32(input));
+  }
+}
+
+void libvisio::VSDParser::readLayer(librevenge::RVNGInputStream *input)
+{
+  libvisio::VSDLayer layer;
+  input->seek(8, librevenge::RVNG_SEEK_CUR);
+  unsigned char colourId = readU8(input);
+  if (colourId == 0xff)
+    input->seek(4, librevenge::RVNG_SEEK_CUR);
+  else
+  {
+    libvisio::Colour colour;
+    colour.r = readU8(input);
+    colour.g = readU8(input);
+    colour.b = readU8(input);
+    colour.a = readU8(input);
+    layer.m_colour = colour;
+  }
+  input->seek(1, librevenge::RVNG_SEEK_CUR);
+  layer.m_visible = !!readU8(input);
+  layer.m_printable = !!readU8(input);
+
+  m_collector->collectLayer(m_header.id, m_header.level, layer);
+}
+
+void libvisio::VSDParser::readLayerMem(librevenge::RVNGInputStream *input)
+{
+  input->seek(13, librevenge::RVNG_SEEK_CUR);
+  unsigned textLength = readU8(input);
+
+  librevenge::RVNGBinaryData  textStream;
+  unsigned long numBytesRead = 0;
+  const unsigned char *tmpBuffer = input->read(textLength*2, numBytesRead);
+  if (numBytesRead)
+  {
+    textStream.append(tmpBuffer, numBytesRead);
+    m_shape.m_layerMem.m_data = textStream;
+    m_shape.m_layerMem.m_format = libvisio::VSD_TEXT_UTF16;
+  }
+
+}
+
 void libvisio::VSDParser::readPage(librevenge::RVNGInputStream *input)
 {
   input->seek(8, librevenge::RVNG_SEEK_CUR); //sub header length and children list length
@@ -965,10 +1117,25 @@ void libvisio::VSDParser::readXFormData(librevenge::RVNGInputStream *input)
   m_shape.m_xform.flipY = !!readU8(input);
 }
 
+void libvisio::VSDParser::readXForm1D(librevenge::RVNGInputStream *input)
+{
+  if (m_shape.m_xform1d)
+    delete m_shape.m_xform1d;
+  m_shape.m_xform1d = new XForm1D();
+  input->seek(1, librevenge::RVNG_SEEK_CUR);
+  m_shape.m_xform1d->beginX = readDouble(input);
+  input->seek(1, librevenge::RVNG_SEEK_CUR);
+  m_shape.m_xform1d->beginY = readDouble(input);
+  input->seek(1, librevenge::RVNG_SEEK_CUR);
+  m_shape.m_xform1d->endX = readDouble(input);
+  input->seek(1, librevenge::RVNG_SEEK_CUR);
+  m_shape.m_xform1d->endY = readDouble(input);
+}
+
 void libvisio::VSDParser::readTxtXForm(librevenge::RVNGInputStream *input)
 {
   if (m_shape.m_txtxform)
-    delete(m_shape.m_txtxform);
+    delete m_shape.m_txtxform;
   m_shape.m_txtxform = new XForm();
   input->seek(1, librevenge::RVNG_SEEK_CUR);
   m_shape.m_txtxform->pinX = readDouble(input);
@@ -1048,17 +1215,21 @@ void libvisio::VSDParser::readPageProps(librevenge::RVNGInputStream *input)
 {
   // Skip bytes representing unit to *display* (value is always inches)
   input->seek(1, librevenge::RVNG_SEEK_CUR);
-  double pageWidth = readDouble(input);
+  const double pageWidth = std::max<double>(readDouble(input), 0);
   input->seek(1, librevenge::RVNG_SEEK_CUR);
-  double pageHeight = readDouble(input);
+  const double pageHeight = std::max<double>(readDouble(input), 0);
   input->seek(1, librevenge::RVNG_SEEK_CUR);
   m_shadowOffsetX = readDouble(input);
   input->seek(1, librevenge::RVNG_SEEK_CUR);
   m_shadowOffsetY = readDouble(input);
   input->seek(1, librevenge::RVNG_SEEK_CUR);
-  double scale = readDouble(input);
+  const double numerator = readDouble(input);
   input->seek(1, librevenge::RVNG_SEEK_CUR);
-  scale /= readDouble(input);
+  double denominator = readDouble(input);
+  if (VSD_ALMOST_ZERO(denominator))
+    denominator = 1;
+
+  const double scale = std::abs(numerator / denominator);
 
   if (m_isStencilStarted && m_currentStencil)
   {
@@ -1109,6 +1280,7 @@ void libvisio::VSDParser::readShape(librevenge::RVNGInputStream *input)
   {
     if (tmpShape->m_foreign)
       m_shape.m_foreign = new ForeignData(*(tmpShape->m_foreign));
+    m_shape.m_tabSets = tmpShape->m_tabSets;
     m_shape.m_text = tmpShape->m_text;
     m_shape.m_textFormat = tmpShape->m_textFormat;
     m_shape.m_misc = tmpShape->m_misc;
@@ -1732,6 +1904,7 @@ void libvisio::VSDParser::readCharIX(librevenge::RVNGInputStream *input)
 
 void libvisio::VSDParser::readParaIX(librevenge::RVNGInputStream *input)
 {
+  long startPosition = input->tell();
   unsigned charCount = readU32(input);
   input->seek(1, librevenge::RVNG_SEEK_CUR);
   double indFirst = readDouble(input);
@@ -1746,12 +1919,55 @@ void libvisio::VSDParser::readParaIX(librevenge::RVNGInputStream *input)
   input->seek(1, librevenge::RVNG_SEEK_CUR);
   double spAfter = readDouble(input);
   unsigned char align = readU8(input);
-  input->seek(26, librevenge::RVNG_SEEK_CUR);
+  unsigned char bullet = readU8(input);
+  input->seek(4, librevenge::RVNG_SEEK_CUR);
+  unsigned fontID = readU16(input);
+  VSDName bulletFont;
+  if (fontID)
+  {
+    std::map<unsigned, VSDName>::const_iterator iter = m_fonts.find(fontID);
+    if (iter != m_fonts.end())
+      bulletFont = iter->second;
+  }
+  input->seek(2, librevenge::RVNG_SEEK_CUR);
+  double bulletFontSize = readDouble(input);
+  input->seek(1, librevenge::RVNG_SEEK_CUR);
+  double textPosAfterBullet = readDouble(input);
   unsigned flags = readU32(input);
+  input->seek(34, librevenge::RVNG_SEEK_CUR);
+  long remainingData = m_header.dataLength - input->tell() + startPosition;
+  unsigned blockLength = 0;
+  VSDName bulletStr;
+
+  while (remainingData >= 4 && (blockLength = readU32(input)))
+  {
+    long blockEnd = blockLength-4 + input->tell();
+    unsigned char blockType = readU8(input);
+    unsigned char blockIdx = readU8(input);
+    if (blockType == 2 && blockIdx == 8)
+    {
+      input->seek(1, librevenge::RVNG_SEEK_CUR);
+      unsigned long numBytes = 2*readU8(input);
+      unsigned long numBytesRead = 0;
+      const unsigned char *tmpBuffer = input->read(numBytes, numBytesRead);
+      if (tmpBuffer && numBytesRead)
+      {
+        librevenge::RVNGBinaryData tmpBulletString(tmpBuffer, numBytesRead);
+        bulletStr = VSDName(tmpBulletString, libvisio::VSD_TEXT_UTF16);
+      }
+    }
+    else if (blockType == 2 && blockIdx == 3)
+    {
+    };
+    input->seek(blockEnd, librevenge::RVNG_SEEK_SET);
+    remainingData -= blockLength;
+  }
+
 
   if (m_isInStyles)
     m_collector->collectParaIXStyle(m_header.id, m_header.level, charCount, indFirst, indLeft, indRight,
-                                    spLine, spBefore, spAfter, align, flags);
+                                    spLine, spBefore, spAfter, align, bullet, bulletStr, bulletFont,
+                                    bulletFontSize, textPosAfterBullet, flags);
   else
   {
     if (m_isStencilStarted)
@@ -1760,9 +1976,13 @@ void libvisio::VSDParser::readParaIX(librevenge::RVNGInputStream *input)
     }
 
     m_shape.m_paraStyle.override(VSDOptionalParaStyle(charCount, indFirst, indLeft, indRight,
-                                                      spLine, spBefore, spAfter, align, flags));
-    m_shape.m_paraList.addParaIX(m_header.id, m_header.level, charCount, indFirst, indLeft, indRight,
-                                 spLine, spBefore, spAfter, align, flags);
+                                                      spLine, spBefore, spAfter, align, bullet,
+                                                      bulletStr, bulletFont, bulletFontSize,
+                                                      textPosAfterBullet, flags));
+    m_shape.m_paraList.addParaIX(m_header.id, m_header.level, charCount, indFirst,
+                                 indLeft, indRight, spLine, spBefore, spAfter, align,
+                                 bullet, bulletStr, bulletFont, bulletFontSize,
+                                 textPosAfterBullet, flags);
   }
 }
 
