@@ -70,14 +70,15 @@ struct VSDThemeReference
 struct VSDOptionalLineStyle
 {
   VSDOptionalLineStyle() :
-    width(), colour(), pattern(), startMarker(), endMarker(), cap() {}
+    width(), colour(), pattern(), startMarker(), endMarker(), cap(), rounding() {}
   VSDOptionalLineStyle(const boost::optional<double> &w, const boost::optional<Colour> &col,
                        const boost::optional<unsigned char> &p, const boost::optional<unsigned char> &sm,
-                       const boost::optional<unsigned char> &em, const boost::optional<unsigned char> &c) :
-    width(w), colour(col), pattern(p), startMarker(sm), endMarker(em), cap(c) {}
+                       const boost::optional<unsigned char> &em, const boost::optional<unsigned char> &c,
+                       const boost::optional<double> &r) :
+    width(w), colour(col), pattern(p), startMarker(sm), endMarker(em), cap(c), rounding(r) {}
   VSDOptionalLineStyle(const VSDOptionalLineStyle &style) :
     width(style.width), colour(style.colour), pattern(style.pattern), startMarker(style.startMarker),
-    endMarker(style.endMarker), cap(style.cap) {}
+    endMarker(style.endMarker), cap(style.cap), rounding(style.rounding) {}
   ~VSDOptionalLineStyle() {}
   void override(const VSDOptionalLineStyle &style)
   {
@@ -87,6 +88,7 @@ struct VSDOptionalLineStyle
     ASSIGN_OPTIONAL(style.startMarker, startMarker);
     ASSIGN_OPTIONAL(style.endMarker, endMarker);
     ASSIGN_OPTIONAL(style.cap, cap);
+    ASSIGN_OPTIONAL(style.rounding, rounding);
   }
 
   boost::optional<double> width;
@@ -95,18 +97,19 @@ struct VSDOptionalLineStyle
   boost::optional<unsigned char> startMarker;
   boost::optional<unsigned char> endMarker;
   boost::optional<unsigned char> cap;
+  boost::optional<double> rounding;
 };
 
 struct VSDLineStyle
 {
   VSDLineStyle() :
-    width(0.01), colour(), pattern(1), startMarker(0), endMarker(0), cap(0) {}
+    width(0.01), colour(), pattern(1), startMarker(0), endMarker(0), cap(0), rounding(0.0) {}
   VSDLineStyle(double w, Colour col, unsigned char p, unsigned char sm,
-               unsigned char em, unsigned char c) :
-    width(w), colour(col), pattern(p), startMarker(sm), endMarker(em), cap(c) {}
+               unsigned char em, unsigned char c, double r) :
+    width(w), colour(col), pattern(p), startMarker(sm), endMarker(em), cap(c), rounding(r) {}
   VSDLineStyle(const VSDLineStyle &style) :
     width(style.width), colour(style.colour), pattern(style.pattern), startMarker(style.startMarker),
-    endMarker(style.endMarker), cap(style.cap) {}
+    endMarker(style.endMarker), cap(style.cap), rounding(style.rounding) {}
   ~VSDLineStyle() {}
   void override(const VSDOptionalLineStyle &style)
   {
@@ -116,6 +119,7 @@ struct VSDLineStyle
     ASSIGN_OPTIONAL(style.startMarker, startMarker);
     ASSIGN_OPTIONAL(style.endMarker, endMarker);
     ASSIGN_OPTIONAL(style.cap, cap);
+    ASSIGN_OPTIONAL(style.rounding, rounding);
   }
 
   double width;
@@ -124,6 +128,7 @@ struct VSDLineStyle
   unsigned char startMarker;
   unsigned char endMarker;
   unsigned char cap;
+  double rounding;
 };
 
 struct VSDOptionalFillStyle
@@ -315,14 +320,20 @@ struct VSDCharStyle
 struct VSDOptionalParaStyle
 {
   VSDOptionalParaStyle() :
-    charCount(0), indFirst(), indLeft(), indRight(), spLine(), spBefore(), spAfter(), align(), flags() {}
+    charCount(0), indFirst(), indLeft(), indRight(), spLine(), spBefore(), spAfter(), align(), bullet(),
+    bulletStr(), bulletFont(), bulletFontSize(0.0), textPosAfterBullet(0.0), flags() {}
   VSDOptionalParaStyle(unsigned cc, const boost::optional<double> &ifst, const boost::optional<double> &il,
                        const boost::optional<double> &ir, const boost::optional<double> &sl, const boost::optional<double> &sb,
-                       const boost::optional<double> &sa, const boost::optional<unsigned char> &a, const boost::optional<unsigned> &f) :
-    charCount(cc), indFirst(ifst), indLeft(il), indRight(ir), spLine(sl), spBefore(sb), spAfter(sa), align(a), flags(f) {}
+                       const boost::optional<double> &sa, const boost::optional<unsigned char> &a, const boost::optional<unsigned char> &b,
+                       const boost::optional<VSDName> &bs, const boost::optional<VSDName> &bf, const boost::optional<double> bfs,
+                       const boost::optional<double> &tpab, const boost::optional<unsigned> &f) :
+    charCount(cc), indFirst(ifst), indLeft(il), indRight(ir), spLine(sl), spBefore(sb), spAfter(sa), align(a), bullet(b),
+    bulletStr(bs), bulletFont(bf), bulletFontSize(bfs), textPosAfterBullet(tpab), flags(f) {}
   VSDOptionalParaStyle(const VSDOptionalParaStyle &style) :
     charCount(style.charCount), indFirst(style.indFirst), indLeft(style.indLeft), indRight(style.indRight), spLine(style.spLine),
-    spBefore(style.spBefore), spAfter(style.spAfter), align(style.align), flags(style.flags) {}
+    spBefore(style.spBefore), spAfter(style.spAfter), align(style.align), bullet(style.bullet), bulletStr(style.bulletStr),
+    bulletFont(style.bulletFont), bulletFontSize(style.bulletFontSize), textPosAfterBullet(style.textPosAfterBullet),
+    flags(style.flags) {}
   ~VSDOptionalParaStyle() {}
   void override(const VSDOptionalParaStyle &style)
   {
@@ -333,6 +344,11 @@ struct VSDOptionalParaStyle
     ASSIGN_OPTIONAL(style.spBefore, spBefore);
     ASSIGN_OPTIONAL(style.spAfter, spAfter);
     ASSIGN_OPTIONAL(style.align, align);
+    ASSIGN_OPTIONAL(style.bullet, bullet);
+    ASSIGN_OPTIONAL(style.bulletStr, bulletStr);
+    ASSIGN_OPTIONAL(style.bulletFont, bulletFont);
+    ASSIGN_OPTIONAL(style.bulletFontSize, bulletFontSize);
+    ASSIGN_OPTIONAL(style.textPosAfterBullet, textPosAfterBullet);
     ASSIGN_OPTIONAL(style.flags, flags);
   }
 
@@ -344,19 +360,29 @@ struct VSDOptionalParaStyle
   boost::optional<double> spBefore;
   boost::optional<double> spAfter;
   boost::optional<unsigned char> align;
+  boost::optional<unsigned char> bullet;
+  boost::optional<VSDName> bulletStr;
+  boost::optional<VSDName> bulletFont;
+  boost::optional<double> bulletFontSize;
+  boost::optional<double> textPosAfterBullet;
   boost::optional<unsigned> flags;
 };
 
 struct VSDParaStyle
 {
   VSDParaStyle() :
-    charCount(0), indFirst(0.0), indLeft(0.0), indRight(0.0), spLine(-1.2), spBefore(0.0), spAfter(0.0), align(1), flags(0) {}
+    charCount(0), indFirst(0.0), indLeft(0.0), indRight(0.0), spLine(-1.2), spBefore(0.0), spAfter(0.0), align(1),
+    bullet(0), bulletStr(), bulletFont(), bulletFontSize(0.0), textPosAfterBullet(0.0), flags(0) {}
   VSDParaStyle(unsigned cc, double ifst, double il, double ir, double sl, double sb,
-               double sa, unsigned char a, unsigned f) :
-    charCount(cc), indFirst(ifst), indLeft(il), indRight(ir), spLine(sl), spBefore(sb), spAfter(sa), align(a), flags(f) {}
+               double sa, unsigned char a, unsigned b, const VSDName &bs,
+               const VSDName &bf, double bfs, double tpab, unsigned f) :
+    charCount(cc), indFirst(ifst), indLeft(il), indRight(ir), spLine(sl), spBefore(sb), spAfter(sa), align(a),
+    bullet(b), bulletStr(bs), bulletFont(bf), bulletFontSize(bfs), textPosAfterBullet(tpab), flags(f) {}
   VSDParaStyle(const VSDParaStyle &style) :
-    charCount(style.charCount), indFirst(style.indFirst), indLeft(style.indLeft), indRight(style.indRight), spLine(style.spLine),
-    spBefore(style.spBefore), spAfter(style.spAfter), align(style.align), flags(style.flags) {}
+    charCount(style.charCount), indFirst(style.indFirst), indLeft(style.indLeft), indRight(style.indRight),
+    spLine(style.spLine), spBefore(style.spBefore), spAfter(style.spAfter), align(style.align),
+    bullet(style.bullet), bulletStr(style.bulletStr), bulletFont(style.bulletFont),
+    bulletFontSize(style.bulletFontSize), textPosAfterBullet(style.textPosAfterBullet), flags(style.flags) {}
   ~VSDParaStyle() {}
   void override(const VSDOptionalParaStyle &style)
   {
@@ -367,6 +393,11 @@ struct VSDParaStyle
     ASSIGN_OPTIONAL(style.spBefore, spBefore);
     ASSIGN_OPTIONAL(style.spAfter, spAfter);
     ASSIGN_OPTIONAL(style.align, align);
+    ASSIGN_OPTIONAL(style.bullet, bullet);
+    ASSIGN_OPTIONAL(style.bulletStr, bulletStr);
+    ASSIGN_OPTIONAL(style.bulletFont, bulletFont);
+    ASSIGN_OPTIONAL(style.bulletFontSize, bulletFontSize);
+    ASSIGN_OPTIONAL(style.textPosAfterBullet, textPosAfterBullet);
     ASSIGN_OPTIONAL(style.flags, flags);
   }
 
@@ -378,6 +409,11 @@ struct VSDParaStyle
   double spBefore;
   double spAfter;
   unsigned char align;
+  unsigned char bullet;
+  VSDName bulletStr;
+  VSDName bulletFont;
+  double bulletFontSize;
+  double textPosAfterBullet;
   unsigned flags;
 };
 
@@ -426,7 +462,7 @@ struct VSDTextBlockStyle
 {
   VSDTextBlockStyle() :
     leftMargin(0.0), rightMargin(0.0), topMargin(0.0), bottomMargin(0.0), verticalAlign(1),
-    isTextBkgndFilled(true), textBkgndColour(0xff,0xff,0xff,0), defaultTabStop(0.5), textDirection(0) {}
+    isTextBkgndFilled(false), textBkgndColour(0xff,0xff,0xff,0), defaultTabStop(0.5), textDirection(0) {}
   VSDTextBlockStyle(double lm, double rm, double tm, double bm, unsigned char va,
                     bool isBgFilled, Colour bgClr, double defTab, unsigned char td) :
     leftMargin(lm), rightMargin(rm), topMargin(tm), bottomMargin(bm), verticalAlign(va),
