@@ -12,18 +12,28 @@
 
 namespace orcus {
 
+extern schema_t SCH_mc;
 extern schema_t SCH_opc_content_types;
 extern schema_t SCH_opc_rels;
 extern schema_t SCH_opc_rels_metadata_core_props;
+extern schema_t SCH_od_rels_calc_chain;
 extern schema_t SCH_od_rels_connections;
 extern schema_t SCH_od_rels_printer_settings;
+extern schema_t SCH_od_rels_rev_headers;
+extern schema_t SCH_od_rels_rev_log;
 extern schema_t SCH_od_rels_shared_strings;
 extern schema_t SCH_od_rels_styles;
 extern schema_t SCH_od_rels_theme;
+extern schema_t SCH_od_rels_usernames;
 extern schema_t SCH_od_rels_worksheet;
 extern schema_t SCH_od_rels_extended_props;
 extern schema_t SCH_od_rels_office_doc;
+extern schema_t SCH_od_rels_table;
+extern schema_t SCH_od_rels_pivot_cache_def;
+extern schema_t SCH_od_rels_pivot_cache_rec;
+extern schema_t SCH_od_rels_pivot_table;
 extern schema_t SCH_xlsx_main;
+extern schema_t SCH_mso_x14ac;
 
 /**
  * Null-terminated array of all schema types.
