@@ -1,29 +1,9 @@
-/*************************************************************************
- *
- * Copyright (c) 2012 Kohei Yoshida
- *
- * Permission is hereby granted, free of charge, to any person
- * obtaining a copy of this software and associated documentation
- * files (the "Software"), to deal in the Software without
- * restriction, including without limitation the rights to use,
- * copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following
- * conditions:
- *
- * The above copyright notice and this permission notice shall be
- * included in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
- * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
- * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
- * OTHER DEALINGS IN THE SOFTWARE.
- *
- ************************************************************************/
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/*
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ */
 
 #include "xml_map_tree.hpp"
 #include "orcus/global.hpp"
@@ -210,7 +190,7 @@ xml_map_tree::attribute::~attribute()
             delete field_ref;
         break;
         default:
-            throw general_error("unexpected reference type in the destructor of attribute.");
+            assert(!"unexpected reference type in the destructor of attribute.");
     }
 }
 
@@ -261,7 +241,7 @@ xml_map_tree::element::~element()
             delete field_ref;
         break;
         default:
-            throw general_error("unexpected reference type in the destructor of element.");
+            assert(!"unexpected reference type in the destructor of element.");
     }
 }
 
@@ -791,3 +771,4 @@ bool operator< (const xml_map_tree::cell_position& left, const xml_map_tree::cel
 
 }
 
+/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
