@@ -1,6 +1,6 @@
 /*************************************************************************
  *
- * Copyright (c) 2012 Kohei Yoshida
+ * Copyright (c) 2012-2014 Kohei Yoshida
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -32,6 +32,7 @@
 #include <mdds/multi_type_vector_trait.hpp>
 #include <mdds/multi_type_vector_custom_func1.hpp>
 #include <mdds/multi_type_vector_custom_func2.hpp>
+#include <mdds/multi_type_vector_custom_func3.hpp>
 
 #include <cassert>
 #include <sstream>
@@ -51,6 +52,7 @@ namespace {
 const mtv::element_t element_type_user_block  = mtv::element_type_user_start;
 const mtv::element_t element_type_muser_block = mtv::element_type_user_start+1;
 const mtv::element_t element_type_fruit_block = mtv::element_type_user_start+2;
+const mtv::element_t element_type_date_block  = mtv::element_type_user_start+3;
 
 enum my_fruit_type { unknown_fruit = 0, apple, orange, mango, peach };
 
@@ -74,6 +76,17 @@ struct muser_cell
     muser_cell() : value(0.0) {}
     muser_cell(const muser_cell& r) : value(r.value) {}
     muser_cell(double _v) : value(_v) {}
+    ~muser_cell() {}
+};
+
+struct date
+{
+    int year;
+    int month;
+    int day;
+
+    date() : year(0), month(0), day(0) {}
+    date(int _year, int _month, int _day) : year(_year), month(_month), day(_day) {}
 };
 
 template<typename T>
@@ -104,17 +117,21 @@ public:
 typedef mdds::mtv::default_element_block<element_type_user_block, user_cell*> user_cell_block;
 typedef mdds::mtv::managed_element_block<element_type_muser_block, muser_cell> muser_cell_block;
 typedef mdds::mtv::default_element_block<element_type_fruit_block, my_fruit_type> fruit_block;
+typedef mdds::mtv::default_element_block<element_type_date_block, date> date_block;
 
 MDDS_MTV_DEFINE_ELEMENT_CALLBACKS_PTR(user_cell, element_type_user_block, NULL, user_cell_block)
 MDDS_MTV_DEFINE_ELEMENT_CALLBACKS_PTR(muser_cell, element_type_muser_block, NULL, muser_cell_block)
 MDDS_MTV_DEFINE_ELEMENT_CALLBACKS(my_fruit_type, element_type_fruit_block, unknown_fruit, fruit_block)
+MDDS_MTV_DEFINE_ELEMENT_CALLBACKS(date, element_type_date_block, date(), date_block)
 
 }
 
 namespace {
 
-typedef multi_type_vector<mtv::custom_block_func2<element_type_user_block, user_cell_block, element_type_muser_block, muser_cell_block> > mtv_type;
-typedef multi_type_vector<mtv::custom_block_func1<element_type_fruit_block, fruit_block> > mtv_fruit_type;
+typedef multi_type_vector<mtv::custom_block_func2<user_cell_block, muser_cell_block> > mtv_type;
+typedef multi_type_vector<mtv::custom_block_func1<fruit_block> > mtv_fruit_type;
+typedef multi_type_vector<
+    mtv::custom_block_func3<muser_cell_block, fruit_block, date_block> > mtv3_type;
 
 template<typename _ColT, typename _ValT>
 bool test_cell_insertion(_ColT& col_db, size_t row, _ValT val)
@@ -153,6 +170,15 @@ void mtv_test_types()
     assert(ct == element_type_fruit_block && ct >= mtv::element_type_user_start);
 }
 
+void mtv_test_block_identifier()
+{
+    stack_printer __stack_printer__("::mtv_test_block_identifier");
+    assert(user_cell_block::block_type == element_type_user_block);
+    assert(muser_cell_block::block_type == element_type_muser_block);
+    assert(fruit_block::block_type == element_type_fruit_block);
+    assert(date_block::block_type == element_type_date_block);
+}
+
 void mtv_test_basic()
 {
     stack_printer __stack_printer__("::mtv_test_basic");
@@ -861,10 +887,10 @@ void mtv_test_managed_block()
     {
         // Release an element.
         mtv_type db(1);
-        muser_cell* p = new muser_cell(4.5);
-        db.set(0, p);
+        muser_cell* p1 = new muser_cell(4.5);
+        db.set(0, p1);
         muser_cell* p2 = db.release<muser_cell*>(0);
-        assert(p == p2);
+        assert(p1 == p2);
         assert(p2->value == 4.5);
         assert(db.is_empty(0));
         delete p2;
@@ -913,6 +939,57 @@ void mtv_test_managed_block()
         db.set(1, new muser_cell(2.2));
         db.set(2, new muser_cell(2.3));
         db.set_empty(0, 2); // Make sure this doesn't release anything.
+
+        // Release with position hint.
+        db = mtv_type(4);
+        db.set(0, new muser_cell(4.5));
+        db.set(1, new muser_cell(4.6));
+        db.set(3, new muser_cell(5.1));
+
+        mtv_type::iterator pos = db.release(0, p1);
+        assert(pos == db.begin());
+        pos = db.release(pos, 3, p2);
+        ++pos;
+        assert(pos == db.end());
+        assert(p1->value == 4.5);
+        assert(p2->value == 5.1);
+        assert(db.block_size() == 3);
+        assert(db.is_empty(0));
+        assert(db.get<muser_cell*>(1)->value == 4.6);
+        assert(db.is_empty(2));
+        assert(db.is_empty(3));
+        delete p1;
+        delete p2;
+    }
+
+    {
+        mtv_type db(5);
+
+        db.set(1, new muser_cell(1.1));
+        db.set(2, new muser_cell(1.2));
+        db.set(3, new muser_cell(1.3));
+
+        db.set(1, 2.1); // Don't leak the overwritten muser_cell instance.
+        db.set(2, 2.2); // ditto
+    }
+
+    {
+        mtv_type db(4);
+        db.set(0, new muser_cell(1.1));
+        db.set(1, new muser_cell(1.2));
+        db.set(2, new muser_cell(1.3));
+
+        db.set(2, 2.1); // Don't leak the overwritten muser_cell instance.
+        db.set(1, 2.0); // ditto
+    }
+
+    {
+        mtv_type db(8);
+        db.set(3, new muser_cell(1.1));
+        db.set(4, new muser_cell(1.2));
+        db.set(5, 1.3);
+
+        db.set(4, 2.2); // Overwrite muser_cell and don't leak.
     }
 }
 
@@ -1317,28 +1394,293 @@ void mtv_test_transfer()
     assert(db2.is_empty(7));
     assert(db2.is_empty(8));
     assert(db2.is_empty(9));
+
+    // Make sure that transfer will overwrite cells in managed blocks.
+    db1 = mtv_type(3);
+    db2 = mtv_type(3);
+    db1.set(0, new muser_cell(1.1));
+    db1.set(1, new muser_cell(1.2));
+    db1.set(2, new muser_cell(1.3));
+
+    db2.set(1, new muser_cell(2.1)); // This element will be overwritten.
+    db1.transfer(0, 2, db2, 0);
+    assert(db1.is_empty(0));
+    assert(db1.is_empty(1));
+    assert(db1.is_empty(2));
+    assert(db2.get<muser_cell*>(0)->value == 1.1);
+    assert(db2.get<muser_cell*>(1)->value == 1.2);
+    assert(db2.get<muser_cell*>(2)->value == 1.3);
+}
+
+void mtv_test_swap()
+{
+    stack_printer __stack_printer__("::mtv_test_swap");
+    mtv3_type db1(5), db2(2);
+    db1.set(0, new muser_cell(1.1));
+    db1.set(1, new muser_cell(1.2));
+    db1.set(2, new muser_cell(1.3));
+    db1.set(3, new muser_cell(1.4));
+    db1.set(4, new muser_cell(1.5));
+
+    db2.set(0, string("A"));
+    db2.set(1, string("B"));
+
+    db1.swap(2, 3, db2, 0);
+
+    // swap blocks of equal size, one managed, and one default.
+
+    db1.clear();
+    db1.resize(10);
+    db2.clear();
+    db2.resize(10);
+
+    db1.set(3, 2.1);
+    db1.set(4, 2.2);
+    db1.set(5, 2.3);
+
+    db2.set(3, new muser_cell(3.1));
+    db2.set(4, new muser_cell(3.2));
+    db2.set(5, new muser_cell(3.3));
+
+    db2.swap(3, 5, db1, 3);
+
+    assert(db1.size() == 10);
+    assert(db1.block_size() == 3);
+    assert(db2.size() == 10);
+    assert(db2.block_size() == 3);
+
+    assert(db1.get<muser_cell*>(3)->value == 3.1);
+    assert(db1.get<muser_cell*>(4)->value == 3.2);
+    assert(db1.get<muser_cell*>(5)->value == 3.3);
+    assert(db2.get<double>(3) == 2.1);
+    assert(db2.get<double>(4) == 2.2);
+    assert(db2.get<double>(5) == 2.3);
+
+    db2.swap(3, 5, db1, 3);
+
+    assert(db1.get<double>(3) == 2.1);
+    assert(db1.get<double>(4) == 2.2);
+    assert(db1.get<double>(5) == 2.3);
+    assert(db2.get<muser_cell*>(3)->value == 3.1);
+    assert(db2.get<muser_cell*>(4)->value == 3.2);
+    assert(db2.get<muser_cell*>(5)->value == 3.3);
+
+    // Same as above, except that the source segment splits the block into 2.
+
+    db1.clear();
+    db1.resize(10);
+    db2.clear();
+    db2.resize(10);
+
+    db1.set(3, 2.1);
+    db1.set(4, 2.2);
+
+    db2.set(3, new muser_cell(3.1));
+    db2.set(4, new muser_cell(3.2));
+    db2.set(5, new muser_cell(3.3));
+
+    db2.swap(3, 4, db1, 3);
+
+    // Another scenario that used to crash on double delete.
+
+    db1.clear();
+    db1.resize(10);
+    db2.clear();
+    db2.resize(10);
+
+    db1.set(2, new muser_cell(4.1));
+    db1.set(3, 4.2);
+    db1.set(4, new muser_cell(4.3));
+
+    db2.set(3, new muser_cell(6.1));
+    db2.set(4, 6.2);
+    db2.set(5, 6.3);
+
+    assert(db1.get<muser_cell*>(2)->value == 4.1);
+    assert(db1.get<double>(3) == 4.2);
+    assert(db1.get<muser_cell*>(4)->value == 4.3);
+
+    assert(db2.get<muser_cell*>(3)->value == 6.1);
+    assert(db2.get<double>(4) == 6.2);
+    assert(db2.get<double>(5) == 6.3);
+
+    db2.swap(4, 4, db1, 4);
+
+    assert(db1.get<muser_cell*>(2)->value == 4.1);
+    assert(db1.get<double>(3) == 4.2);
+    assert(db1.get<double>(4) == 6.2);
+
+    assert(db2.get<muser_cell*>(3)->value == 6.1);
+    assert(db2.get<muser_cell*>(4)->value == 4.3);
+    assert(db2.get<double>(5) == 6.3);
+
+    // One more on double deletion...
+
+    db1.clear();
+    db1.resize(10);
+    db2.clear();
+    db2.resize(10);
+
+    db1.set(0, 2.1);
+    db1.set(1, 2.2);
+    db1.set(2, 2.3);
+    db1.set(3, new muser_cell(4.5));
+
+    db2.set(2, new muser_cell(3.1));
+    db2.set(3, new muser_cell(3.2));
+    db2.set(4, new muser_cell(3.3));
+
+    db1.swap(2, 2, db2, 3);
+
+    assert(db1.get<double>(0) == 2.1);
+    assert(db1.get<double>(1) == 2.2);
+    assert(db1.get<muser_cell*>(2)->value == 3.2);
+    assert(db1.get<muser_cell*>(3)->value == 4.5);
+
+    assert(db2.get<muser_cell*>(2)->value == 3.1);
+    assert(db2.get<double>(3) == 2.3);
+    assert(db2.get<muser_cell*>(4)->value == 3.3);
+
+    assert(db1.check_block_integrity());
+    assert(db2.check_block_integrity());
+}
+
+void mtv_test_custom_block_func3()
+{
+    stack_printer __stack_printer__("::mtv_test_custom_block_func3");
+    mtv3_type db(10);
+
+    // Insert custom elements.
+    db.set(0, new muser_cell(12.3));
+    db.set(1, apple);
+    db.set(2, date(1989,12,13));
+    db.set(3, date(2011,8,7));
+    assert(db.get_type(0) == element_type_muser_block);
+    assert(db.get_type(1) == element_type_fruit_block);
+    assert(db.get_type(2) == element_type_date_block);
+    assert(db.get_type(3) == element_type_date_block);
+    assert(db.get<muser_cell*>(0)->value == 12.3);
+    assert(db.get<my_fruit_type>(1) == apple);
+    assert(db.get<date>(2).year == 1989);
+    assert(db.get<date>(2).month == 12);
+    assert(db.get<date>(2).day == 13);
+    assert(db.get<date>(3).year == 2011);
+    assert(db.get<date>(3).month == 8);
+    assert(db.get<date>(3).day == 7);
+    assert(db.block_size() == 4);
+
+    // We should still support the primitive types.
+    db.set(8, 34.56);
+    assert(db.get<double>(8) == 34.56);
+}
+
+void mtv_test_release()
+{
+    stack_printer __stack_printer__("::mtv_test_release");
+    mtv_type db(3);
+    muser_cell c1(1.2), c2(1.3); // allocated on the stack.
+    db.set(0, &c1);
+    db.set(2, &c2);
+    db.release(); // Prevent invalid free when db goes out of scope.
+
+    // Variant with no argument should make the container empty after the call.
+    assert(db.empty());
+
+    db.push_back(new muser_cell(10.0));
+    db.push_back(&c1);
+    db.push_back(&c2);
+    db.push_back(new muser_cell(10.1));
+
+    assert(db.size() == 4);
+    assert(db.block_size() == 1);
+
+    // Release those allocated on the stack to avoid double deletion.
+    mtv_type::iterator it = db.release_range(1, 2);
+
+    // Check the integrity of the returned iterator.
+    assert(it->type == mtv::element_type_empty);
+    assert(it->size == 2);
+    mtv_type::iterator check = it;
+    --check;
+    assert(check == db.begin());
+    check = it;
+    ++check;
+    ++check;
+    assert(check == db.end());
+
+    db.push_back(new muser_cell(10.2));
+    assert(db.size() == 5);
+
+    muser_cell* p1 = db.get<muser_cell*>(3);
+    muser_cell* p2 = db.get<muser_cell*>(4);
+    assert(p1->value == 10.1);
+    assert(p2->value == 10.2);
+
+    // Pass iterator as a position hint.
+    it = db.release_range(it, 3, 4);
+    assert(db.block_size() == 2);
+
+    // Check the returned iterator.
+    check = it;
+    --check;
+    assert(check == db.begin());
+    assert(check->type == element_type_muser_block);
+    assert(check->size == 1);
+    assert(it->type == mtv::element_type_empty);
+    assert(it->size == 4);
+    ++it;
+    assert(it == db.end());
+
+    // It should be safe to delete these instances now that they've been released.
+    delete p1;
+    delete p2;
+}
+
+void mtv_test_construction_with_array()
+{
+    stack_printer __stack_printer__("::mtv_test_construction_with_array");
+    {
+        std::vector<muser_cell*> vals;
+        vals.push_back(new muser_cell(2.1));
+        vals.push_back(new muser_cell(2.2));
+        vals.push_back(new muser_cell(2.3));
+        mtv_type db(vals.size(), vals.begin(), vals.end());
+
+        db.set(1, 10.2); // overwrite.
+        assert(db.size() == 3);
+        assert(db.block_size() == 3);
+        assert(db.get<muser_cell*>(0)->value == 2.1);
+        assert(db.get<double>(1) == 10.2);
+        assert(db.get<muser_cell*>(2)->value == 2.3);
+
+        // Now those heap objects are owned by the container.  Clearing the
+        // array shouldn't leak.
+        vals.clear();
+    }
 }
 
 }
 
 int main (int argc, char **argv)
 {
-    cmd_options opt;
-    if (!parse_cmd_options(argc, argv, opt))
-        return EXIT_FAILURE;
-
-    if (opt.test_func)
+    try
     {
         mtv_test_types();
+        mtv_test_block_identifier();
         mtv_test_basic();
         mtv_test_equality();
         mtv_test_managed_block();
         mtv_test_custom_block_func1();
         mtv_test_transfer();
+        mtv_test_swap();
+        mtv_test_custom_block_func3();
+        mtv_test_release();
+        mtv_test_construction_with_array();
     }
-
-    if (opt.test_perf)
+    catch (const std::exception& e)
     {
+        cout << "Test failed: " << e.what() << endl;
+        return EXIT_FAILURE;
     }
 
     cout << "Test finished successfully!" << endl;
