@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2012, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2014, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -41,7 +41,7 @@ void set_binmode(FILE *stream)
 #  ifdef __HIGHC__
   _setmode(stream, O_BINARY);
 #  else
-  setmode(fileno(stream), O_BINARY);
+  (void)setmode(fileno(stream), O_BINARY);
 #  endif
 #else
   (void)stream;
