@@ -1,26 +1,28 @@
 /*
-  Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
 
-  The MySQL Connector/C++ is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+The MySQL Connector/C++ is licensed under the terms of the GPLv2
+<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+MySQL Connectors. There are special exceptions to the terms and
+conditions of the GPLv2 as it is applied to this software, see the
+FLOSS License Exception
+<http://www.mysql.com/about/legal/licensing/foss-exception.html>.
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published
+by the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
 
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
- */
 
 #include "unit_fixture.h"
 #include <cstdio>
@@ -323,8 +325,7 @@ void unit_fixture::createSchemaObject(String object_type, String object_name,
   dropSchemaObject(object_type, object_name);
 
   String sql("CREATE  ");
-  sql.resize(object_name.length()
-             + object_type.length() + columns_and_other_stuff.length() + 10);
+
   sql.append(object_type);
   sql.append(" ");
   sql.append(object_name);
@@ -345,12 +346,15 @@ void unit_fixture::createTable(String table_name, String columns_and_other_stuff
   createSchemaObject("TABLE", table_name, columns_and_other_stuff);
 }
 
+
 void unit_fixture::dropTable(String table_name)
 {
   dropSchemaObject("TABLE", table_name);
 }
 
-sql::Connection * unit_fixture::getConnection()
+
+sql::Connection *
+unit_fixture::getConnection(sql::ConnectOptionsMap *additional_options)
 {
   if (driver == NULL)
   {
@@ -373,6 +377,15 @@ sql::Connection * unit_fixture::getConnection()
     logMsg("Connection using dynamic load of clientlib " DYNLOAD_MYSQL_LIB);
   }
 
+  if (additional_options != NULL)
+  {
+    for (sql::ConnectOptionsMap::const_iterator cit= additional_options->begin();
+         cit != additional_options->end(); ++cit)
+    {
+      connection_properties[cit->first]= cit->second;
+    }
+  }
+
   return driver->connect(connection_properties);
 }
 
@@ -386,11 +399,14 @@ void unit_fixture::logErr(const String & message)
   TestsListener::errorsLog(message + "\n");
 }
 
+
 void unit_fixture::logDebug(const String & message)
 {
   logMsg(message);
 }
 
+
+/* There is not really need to have it as a class method */
 int unit_fixture::getMySQLVersion(Connection & con)
 {
   DatabaseMetaData * dbmeta=con->getMetaData();
